@@ -5,7 +5,7 @@
 // Simple demo program for Magick++
 //
 // Concept and algorithms lifted from PerlMagick demo script written
-// by John Christy.
+// by Cristy.
 //
 // Max run-time size 60MB (as compared with 95MB for PerlMagick) under SPARC Solaris
 //
@@ -24,6 +24,9 @@ int main( int /*argc*/, char ** argv)
 
   // Initialize ImageMagick install location for Windows
   InitializeMagick(*argv);
+
+  const char *const p = getenv("MAGICK_FONT");
+  const string MAGICK_FONT(p ? p : "");
   
   try {
     
@@ -82,8 +85,7 @@ int main( int /*argc*/, char ** argv)
       example.label( "Annotate" );
       example.density( "72x72" );
       example.fontPointsize( 18 );
-      if (getenv("MAGICK_FONT") != 0)
-        example.font(string(getenv("MAGICK_FONT")));
+      example.font(MAGICK_FONT);
       example.strokeColor( Color() );
       example.fillColor( "gold" );
       example.annotate( "Magick++", "+0+20", NorthGravity );
