@@ -609,9 +609,12 @@ void Magick::Options::strokeDashArray(const double *strokeDashArray_)
       if (!_drawInfo->dash_pattern)
         throwExceptionExplicit(MagickCore::ResourceLimitError,
           "Unable to allocate dash-pattern memory");
-      // Copy elements
-      memcpy(_drawInfo->dash_pattern,strokeDashArray_,(x+1)*sizeof(double));
-      _drawInfo->dash_pattern[x]=0.0;
+      else
+        {
+          // Copy elements
+          memcpy(_drawInfo->dash_pattern,strokeDashArray_,(x+1)*sizeof(double));
+          _drawInfo->dash_pattern[x]=0.0;
+        }
     }
 }
 
