@@ -17,7 +17,7 @@
 %                                 July 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,6 +47,7 @@
 #include "magick/image.h"
 #include "magick/image-private.h"
 #include "magick/list.h"
+#include "magick/locale-private.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/module.h"
@@ -140,17 +141,16 @@ static MagickBooleanType IsMETA(const unsigned char *magick,const size_t length)
 %    o exception: return any errors or warnings in this structure.
 %
 */
-typedef struct _html_code
+
+static const struct
 {
-  const short
+  const unsigned char
     len;
 
   const char
-    *code,
+    code[7],
     val;
-} html_code;
-
-static const html_code html_codes[] = {
+} html_codes[] = {
 #ifdef HANDLE_GT_LT
   { 4,"&lt;",'<' },
   { 4,"&gt;",'>' },
@@ -162,7 +162,7 @@ static const html_code html_codes[] = {
 
 static int stringnicmp(const char *p,const char *q,size_t n)
 {
-  register ssize_t
+  ssize_t
     i,
     j;
 
@@ -177,11 +177,11 @@ static int stringnicmp(const char *p,const char *q,size_t n)
     if ((*p == '\0') || (*q == '\0'))
       break;
     i=(*p);
-    if (islower(i))
-      i=toupper(i);
+    if (islower((int) ((unsigned char) i)) != 0)
+      i=LocaleToUppercase(i);
     j=(*q);
-    if (islower(j))
-      j=toupper(j);
+    if (islower((int) ((unsigned char) j)) != 0)
+      j=LocaleToUppercase(j);
     if (i != j)
       break;
     n--;
@@ -190,7 +190,7 @@ static int stringnicmp(const char *p,const char *q,size_t n)
     p++;
     q++;
   }
-  return(toupper((int) *p)-toupper((int) *q));
+  return(LocaleToUppercase((int) *p)-LocaleToUppercase((int) *q));
 }
 
 static size_t convertHTMLcodes(char *s)
@@ -198,7 +198,7 @@ static size_t convertHTMLcodes(char *s)
   int
     value;
 
-  register size_t
+  size_t
     i;
 
   size_t
@@ -230,7 +230,7 @@ static size_t convertHTMLcodes(char *s)
       *s=value;
       return(o);
     }
-  for (i=0; i < (ssize_t) (sizeof(html_codes)/sizeof(html_code)); i++)
+  for (i=0; i < (ssize_t) (sizeof(html_codes)/sizeof(html_codes[0])); i++)
   {
     if (html_codes[i].len <= (ssize_t) length)
       if (stringnicmp(s,html_codes[i].code,(size_t) (html_codes[i].len)) == 0)
@@ -243,10 +243,12 @@ static size_t convertHTMLcodes(char *s)
   return(0);
 }
 
-static char *super_fgets(char **b, int *blen, Image *file)
+static char *super_fgets(char **b, size_t *blen, Image *file)
 {
   int
-    c,
+    c;
+
+  size_t
     len;
 
   unsigned char
@@ -260,34 +262,41 @@ static char *super_fgets(char **b, int *blen, Image *file)
     c=ReadBlobByte(file);
     if (c == EOF || c == '\n')
       break;
-    if ((q-p+1) >= (int) len)
+    if ((size_t) (q-p+1) >= len)
       {
-        int
+        size_t
           tlen;
 
-        tlen=q-p;
+        unsigned char
+          *buffer;
+
+        tlen=(size_t) (q-p);
         len<<=1;
-        p=(unsigned char *) ResizeQuantumMemory(p,(size_t) len+2UL,sizeof(*p));
-        *b=(char *) p;
-        if (p == (unsigned char *) NULL)
-          break;
+        buffer=(unsigned char *) ResizeQuantumMemory(p,len+2UL,sizeof(*p));
+        if (buffer == (unsigned char *) NULL)
+          {
+            p=(unsigned char *) RelinquishMagickMemory(p);
+            break;
+          }
+        p=buffer;
         q=p+tlen;
       }
     *q=(unsigned char) c;
   }
+  *b=(char *) p;
   *blen=0;
   if (p != (unsigned char *) NULL)
     {
-      int
+      size_t
         tlen;
 
-      tlen=q-p;
+      tlen=(size_t) (q-p);
       if (tlen == 0)
         return (char *) NULL;
       p[tlen] = '\0';
       *blen=++tlen;
     }
-  return((char *) p);
+  return(*b);
 }
 
 #define IPTC_ID 1028
@@ -313,13 +322,13 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
   unsigned int
     recnum;
 
-  int
-    inputlen = MaxTextExtent;
-
   MagickOffsetType
     savedpos,
     currentpos;
 
+  size_t
+    inputlen = MaxTextExtent;
+
   ssize_t
     savedolen = 0L,
     outputlen = 0L;
@@ -340,13 +349,13 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
     state=0;
     next=0;
 
-    token=(char *) AcquireQuantumMemory((size_t) inputlen,sizeof(*token));
+    token=(char *) AcquireQuantumMemory(inputlen,sizeof(*token));
     if (token == (char *) NULL)
       break;
-    newstr=(char *) AcquireQuantumMemory((size_t) inputlen,sizeof(*newstr));
+    newstr=(char *) AcquireQuantumMemory(inputlen,sizeof(*newstr));
     if (newstr == (char *) NULL)
       break;
-    while (Tokenizer(token_info,0,token,(size_t) inputlen,line,"","=","\"",0,
+    while (Tokenizer(token_info,0,token,inputlen,line,"","=","\"",0,
            &brkused,&next,&quoted)==0)
     {
       if (state == 0)
@@ -361,7 +370,7 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
 
           state=0;
           next=0;
-          while (Tokenizer(token_info,0,newstr,(size_t) inputlen,token,"","#",
+          while (Tokenizer(token_info,0,newstr,inputlen,token,"","#",
             "", 0,&brkused,&next,&quoted)==0)
           {
             switch (state)
@@ -376,11 +385,13 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
                 recnum = (unsigned int) StringToUnsignedLong(newstr);
                 break;
               case 2:
-                name=(char *) AcquireQuantumMemory(strlen(newstr)+MaxTextExtent,
-                  sizeof(*name));
-                if (name)
-                  (void) strcpy(name,newstr);
+              {
+                size_t extent = strlen(newstr)+MagickPathExtent;
+                name=(char *) AcquireQuantumMemory(extent,sizeof(*name));
+                if (name != (char *) NULL)
+                  (void) CopyMagickString(name,newstr,extent);
                 break;
+              }
             }
             state++;
           }
@@ -400,7 +411,7 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
 
             next=0;
             len = (ssize_t) strlen(token);
-            while (Tokenizer(token_info,0,newstr,(size_t) inputlen,token,"","&",
+            while (Tokenizer(token_info,0,newstr,inputlen,token,"","&",
               "",0,&brkused,&next,&quoted)==0)
             {
               if (brkused && next > 0)
@@ -556,7 +567,7 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
   return(outputlen);
 }
 
-static char *super_fgets_w(char **b, int *blen, Image *file)
+static char *super_fgets_w(char **b, size_t *blen, Image *file)
 {
   int
     c,
@@ -577,32 +588,39 @@ static char *super_fgets_w(char **b, int *blen, Image *file)
       break;
    if ((q-p+1) >= (int) len)
       {
-        int
+        size_t
           tlen;
 
-        tlen=q-p;
+        unsigned char
+          *buffer;
+
+        tlen=(size_t) (q-p);
         len<<=1;
-        p=(unsigned char *) ResizeQuantumMemory(p,(size_t) (len+2),sizeof(*p));
-        *b=(char *) p;
-        if (p == (unsigned char *) NULL)
-          break;
+        buffer=(unsigned char *) ResizeQuantumMemory(p,len+2,sizeof(*p));
+        if (buffer == (unsigned char *) NULL)
+          {
+            p=(unsigned char *) RelinquishMagickMemory(p);
+            break;
+          }
+        p=buffer;
         q=p+tlen;
       }
     *q=(unsigned char) c;
   }
+  *b=(char *) p;
   *blen=0;
   if ((*b) != (char *) NULL)
     {
-      int
+      size_t
         tlen;
 
-      tlen=q-p;
+      tlen=(size_t) (q-p);
       if (tlen == 0)
         return (char *) NULL;
       p[tlen] = '\0';
       *blen=++tlen;
     }
-  return((char *) p);
+  return(*b);
 }
 
 static ssize_t parse8BIMW(Image *ifile, Image *ofile)
@@ -625,7 +643,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
   unsigned int
     recnum;
 
-  int
+  size_t
     inputlen = MaxTextExtent;
 
   ssize_t
@@ -641,7 +659,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
 
   dataset = 0;
   recnum = 0;
-  line=(char *) AcquireQuantumMemory((size_t) inputlen,sizeof(*line));
+  line=(char *) AcquireQuantumMemory(inputlen,sizeof(*line));
   if (line == (char *) NULL)
     return(-1);
   newstr = name = token = (char *) NULL;
@@ -652,13 +670,13 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
     state=0;
     next=0;
 
-    token=(char *) AcquireQuantumMemory((size_t) inputlen,sizeof(*token));
+    token=(char *) AcquireQuantumMemory(inputlen,sizeof(*token));
     if (token == (char *) NULL)
       break;
-    newstr=(char *) AcquireQuantumMemory((size_t) inputlen,sizeof(*newstr));
+    newstr=(char *) AcquireQuantumMemory(inputlen,sizeof(*newstr));
     if (newstr == (char *) NULL)
       break;
-    while (Tokenizer(token_info,0,token,(size_t) inputlen,line,"","=","\"",0,
+    while (Tokenizer(token_info,0,token,inputlen,line,"","=","\"",0,
       &brkused,&next,&quoted)==0)
     {
       if (state == 0)
@@ -673,7 +691,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
 
           state=0;
           next=0;
-          while (Tokenizer(token_info,0,newstr,(size_t) inputlen,token,"","#",
+          while (Tokenizer(token_info,0,newstr,inputlen,token,"","#",
             "",0,&brkused,&next,&quoted)==0)
           {
             switch (state)
@@ -712,7 +730,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
 
             next=0;
             len = (ssize_t) strlen(token);
-            while (Tokenizer(token_info,0,newstr,(size_t) inputlen,token,"","&",
+            while (Tokenizer(token_info,0,newstr,inputlen,token,"","&",
               "",0,&brkused,&next,&quoted)==0)
             {
               if (brkused && next > 0)
@@ -1147,6 +1165,7 @@ static void CopyBlob(Image *source,Image *destination)
     sizeof(*buffer));
   if (buffer != (unsigned char *) NULL)
     {
+      (void) memset(buffer,0,MagickMaxBufferExtent*sizeof(*buffer));
       i=0;
       while ((length=ReadBlob(source,MagickMaxBufferExtent,buffer)) != 0)
       {
@@ -1180,7 +1199,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   size_t
     length;
 
-  void
+  unsigned char
     *blob;
 
   /*
@@ -1188,11 +1207,11 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -1230,7 +1249,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           length=(size_t) parse8BIM(image, buff);
           if (length == 0)
             {
-              blob=DetachBlob(buff->blob);
+              blob=(unsigned char *) DetachBlob(buff->blob);
               blob=(unsigned char *) RelinquishMagickMemory(blob);
               buff=DestroyImage(buff);
               ThrowReaderException(CorruptImageError,"CorruptImage");
@@ -1243,7 +1262,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           length=(size_t) parse8BIMW(image, buff);
           if (length == 0)
             {
-              blob=DetachBlob(buff->blob);
+              blob=(unsigned char *) DetachBlob(buff->blob);
               blob=(unsigned char *) RelinquishMagickMemory(blob);
               buff=DestroyImage(buff);
               ThrowReaderException(CorruptImageError,"CorruptImage");
@@ -1257,14 +1276,14 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         GetBlobSize(buff));
       if (profile == (StringInfo *) NULL)
         {
-          blob=DetachBlob(buff->blob);
+          blob=(unsigned char *) DetachBlob(buff->blob);
           blob=(unsigned char *) RelinquishMagickMemory(blob);
           buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       status=SetImageProfile(image,"8bim",profile);
       profile=DestroyStringInfo(profile);
-      blob=DetachBlob(buff->blob);
+      blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
       if (status == MagickFalse)
@@ -1296,7 +1315,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
 
           if (image_info->profile == (void *) NULL)
             {
-              blob=DetachBlob(buff->blob);
+              blob=(unsigned char *) DetachBlob(buff->blob);
               blob=(unsigned char *) RelinquishMagickMemory(blob);
               buff=DestroyImage(buff);
               ThrowReaderException(CoderError,"NoIPTCProfileAvailable");
@@ -1305,7 +1324,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           iptc=AcquireImage((ImageInfo *) NULL);
           if (iptc == (Image *) NULL)
             {
-              blob=DetachBlob(buff->blob);
+              blob=(unsigned char *) DetachBlob(buff->blob);
               blob=(unsigned char *) RelinquishMagickMemory(blob);
               buff=DestroyImage(buff);
               ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -1313,7 +1332,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
           AttachBlob(iptc->blob,GetStringInfoDatum(profile),
             GetStringInfoLength(profile));
           result=jpeg_embed(image,buff,iptc);
-          blob=DetachBlob(iptc->blob);
+          blob=(unsigned char *) DetachBlob(iptc->blob);
           blob=(unsigned char *) RelinquishMagickMemory(blob);
           iptc=DestroyImage(iptc);
           if (result == 0)
@@ -1327,7 +1346,7 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       status=SetImageProfile(image,name,profile);
       profile=DestroyStringInfo(profile);
-      blob=DetachBlob(buff->blob);
+      blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
       if (status == MagickFalse)
@@ -1354,14 +1373,14 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         GetBlobSize(buff));
       if (profile == (StringInfo *) NULL)
         {
-          blob=DetachBlob(buff->blob);
+          blob=(unsigned char *) DetachBlob(buff->blob);
           blob=(unsigned char *) RelinquishMagickMemory(blob);
           buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       (void) SetImageProfile(image,"icc",profile);
       profile=DestroyStringInfo(profile);
-      blob=DetachBlob(buff->blob);
+      blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
     }
@@ -1382,14 +1401,14 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         GetBlobSize(buff));
       if (profile == (StringInfo *) NULL)
         {
-          blob=DetachBlob(buff->blob);
+          blob=(unsigned char *) DetachBlob(buff->blob);
           blob=(unsigned char *) RelinquishMagickMemory(blob);
           buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       (void) SetImageProfile(image,"iptc",profile);
       profile=DestroyStringInfo(profile);
-      blob=DetachBlob(buff->blob);
+      blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
     }
@@ -1410,14 +1429,14 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         GetBlobSize(buff));
       if (profile == (StringInfo *) NULL)
         {
-          blob=DetachBlob(buff->blob);
+          blob=(unsigned char *) DetachBlob(buff->blob);
           blob=(unsigned char *) RelinquishMagickMemory(blob);
           buff=DestroyImage(buff);
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       (void) SetImageProfile(image,"xmp",profile);
       profile=DestroyStringInfo(profile);
-      blob=DetachBlob(buff->blob);
+      blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
     }
@@ -1460,7 +1479,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Photoshop resource format");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("8BIMTEXT");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1469,7 +1488,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Photoshop resource text format");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("8BIMWTEXT");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1478,7 +1497,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Photoshop resource wide text format");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("APP1");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1487,7 +1506,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Raw application information");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("APP1JPEG");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1496,7 +1515,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Raw JPEG binary data");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("EXIF");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1505,7 +1524,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Exif digital camera binary data");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XMP");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1514,7 +1533,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Adobe XML metadata");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ICM");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1523,7 +1542,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("ICC Color Profile");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("ICC");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1532,7 +1551,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("ICC Color Profile");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("IPTC");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1541,7 +1560,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("IPTC Newsphoto");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("IPTCTEXT");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1550,7 +1569,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("IPTC Newsphoto text format");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("IPTCWTEXT");
   entry->decoder=(DecodeImageHandler *) ReadMETAImage;
@@ -1559,7 +1578,7 @@ ModuleExport size_t RegisterMETAImage(void)
   entry->stealth=MagickTrue;
   entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("IPTC Newsphoto text format");
-  entry->module=ConstantString("META");
+  entry->magick_module=ConstantString("META");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -1633,10 +1652,10 @@ static size_t GetIPTCStream(unsigned char **info,size_t length)
   int
     c;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -1793,7 +1812,7 @@ iptc_find:
   return(info_length);
 }
 
-static void formatString(Image *ofile, const char *s, int len)
+static void formatString(Image *ofile, const char *s, ssize_t len)
 {
   char
     temp[MaxTextExtent];
@@ -1817,7 +1836,7 @@ static void formatString(Image *ofile, const char *s, int len)
       (void) WriteBlobString(ofile,"&quot;");
       break;
     default:
-      if (isprint(c))
+      if (isprint((int) ((unsigned char) c)) != 0)
         (void) WriteBlobByte(ofile,(unsigned char) *s);
       else
         {
@@ -1927,7 +1946,7 @@ static int formatIPTC(Image *ifile, Image *ofile)
 
   int
     i,
-    tagcount = (int) (sizeof(tags) / sizeof(tag_spec));
+    tagcount = (int) (sizeof(tags) / sizeof(tags[0]));
 
   int
     c;
@@ -2070,7 +2089,7 @@ static int formatIPTCfromBuffer(Image *ofile, char *s, ssize_t len)
 
   int
     i,
-    tagcount = (int) (sizeof(tags) / sizeof(tag_spec));
+    tagcount = (int) (sizeof(tags) / sizeof(tags[0]));
 
   int
     c;
@@ -2323,7 +2342,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   length=0;
   if (LocaleCompare(image_info->magick,"8BIM") == 0)
@@ -2356,6 +2375,8 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       if (profile == (StringInfo *) NULL)
         ThrowWriterException(CoderError,"No8BIMDataIsAvailable");
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+      if (status == MagickFalse)
+        return(status);
       info=GetStringInfoDatum(profile);
       length=GetStringInfoLength(profile);
       length=GetIPTCStream(&info,length);
