@@ -18,7 +18,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -67,11 +67,14 @@
 #  endif
 #  include <libxml/parser.h>
 #  include <libxml/xmlmemory.h>
+#if defined(LIBXML_FTP_ENABLED)
 #  include <libxml/nanoftp.h>
+#endif
+#if defined(LIBXML_HTTP_ENABLED)
 #  include <libxml/nanohttp.h>
 #endif
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !defined(__MINGW32__)
+#endif
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
 #  include <urlmon.h>
 #  pragma comment(lib, "urlmon.lib")
 #endif
@@ -141,7 +144,9 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *file;
 
   Image
-    *image;
+    *image,
+    *images,
+    *next;
 
   ImageInfo
     *read_info;
@@ -149,16 +154,47 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   int
     unique_file;
 
+  images=(Image *) NULL;
+  image=AcquireImage(image_info);
   read_info=CloneImageInfo(image_info);
   SetImageInfoBlob(read_info,(void *) NULL,0);
+#if !defined(MAGICKCORE_WINDOWS_SUPPORT)
+  if (LocaleCompare(read_info->magick,"https") == 0)
+    {
+      MagickBooleanType
+        status;
+
+      /*
+        Leverage delegate to read HTTPS link.
+      */
+      status=InvokeDelegate(read_info,image,"https:decode",(char *) NULL,
+        exception);
+      if (status != MagickFalse)
+        {
+          (void) FormatLocaleString(read_info->filename,MagickPathExtent,
+            "%s.dat",read_info->unique);
+          *read_info->magick='\0';
+          images=ReadImage(read_info,exception);
+          (void) RelinquishUniqueFileResource(read_info->filename);
+          if (images != (Image *) NULL)
+            for (next=images; next != (Image *) NULL; next=next->next)
+              (void) CopyMagickString(next->filename,image->filename,
+                MaxTextExtent);
+        }
+      read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
+      return(images);
+    }
+#endif
   if (LocaleCompare(read_info->magick,"file") == 0)
     {
       (void) CopyMagickString(read_info->filename,image_info->filename+2,
         MaxTextExtent);
       *read_info->magick='\0';
-      image=ReadImage(read_info,exception);
+      images=ReadImage(read_info,exception);
       read_info=DestroyImageInfo(read_info);
-      return(GetFirstImageInList(image));
+      image=DestroyImage(image);
+      return(GetFirstImageInList(images));
     }
   file=(FILE *) NULL;
   unique_file=AcquireUniqueFileResource(read_info->filename);
@@ -169,14 +205,14 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
         read_info->filename);
       read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
   (void) CopyMagickString(filename,image_info->magick,MaxTextExtent);
   (void) ConcatenateMagickString(filename,":",MaxTextExtent);
   LocaleLower(filename);
   (void) ConcatenateMagickString(filename,image_info->filename,MaxTextExtent);
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !defined(__MINGW32__)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
   (void) fclose(file);
   if (URLDownloadToFile(NULL,filename,read_info->filename,0,NULL) != S_OK)
     {
@@ -184,6 +220,7 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
         filename);
       (void) RelinquishUniqueFileResource(read_info->filename);
       read_info=DestroyImageInfo(read_info);
+      image=DestroyImage(image);
       return((Image *) NULL);
     }
 #else
@@ -237,18 +274,22 @@ static Image *ReadURLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   (void) fclose(file);
 #endif
   *read_info->magick='\0';
-  image=ReadImage(read_info,exception);
+  images=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  if (images != (Image *) NULL)
+    for (next=images; next != (Image *) NULL; next=next->next)
+      (void) CopyMagickString(next->filename,image->filename,MaxTextExtent);
   read_info=DestroyImageInfo(read_info);
-  if (image != (Image *) NULL)
-    GetPathComponent(image_info->filename,TailPath,image->filename);
+  if (images != (Image *) NULL)
+    GetPathComponent(image_info->filename,TailPath,images->filename);
   else
     {
       (void) ThrowMagickException(exception,GetMagickModule(),CoderError,
         "NoDataReturned","`%s'",filename);
       return((Image *) NULL);
     }
-  return(GetFirstImageInList(image));
+  image=DestroyImage(image);
+  return(GetFirstImageInList(images));
 }
 
 /*
@@ -280,38 +321,33 @@ ModuleExport size_t RegisterURLImage(void)
     *entry;
 
   entry=SetMagickInfo("HTTP");
-#if (defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !defined(__MINGW32__)) || \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || \
     (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_HTTP_ENABLED))
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
   entry->description=ConstantString("Uniform Resource Locator (http://)");
-  entry->module=ConstantString("URL");
+  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("HTTPS");
-#if defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !defined(__MINGW32__)
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
-#endif
   entry->description=ConstantString("Uniform Resource Locator (https://)");
-  entry->module=ConstantString("URL");
+  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FTP");
-#if (defined(MAGICKCORE_WINDOWS_SUPPORT) && \
-    !defined(__MINGW32__)) || \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || \
     (defined(MAGICKCORE_XML_DELEGATE) && defined(LIBXML_FTP_ENABLED))
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
 #endif
   entry->description=ConstantString("Uniform Resource Locator (ftp://)");
-  entry->module=ConstantString("URL");
+  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("FILE");
   entry->decoder=(DecodeImageHandler *) ReadURLImage;
   entry->description=ConstantString("Uniform Resource Locator (file://)");
-  entry->module=ConstantString("URL");
+  entry->magick_module=ConstantString("URL");
   entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
