@@ -17,7 +17,7 @@
 %                                October 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -133,7 +133,7 @@ typedef struct _EdgeInfo
     bottom;
 } EdgeInfo;
 
-static double GetEdgeBackgroundFactor(const Image *image,
+static double GetEdgeBackgroundCensus(const Image *image,
   const CacheView *image_view,const GravityType gravity,const size_t width,
   const size_t height,const ssize_t x_offset,const ssize_t y_offset,
   ExceptionInfo *exception)
@@ -144,8 +144,11 @@ static double GetEdgeBackgroundFactor(const Image *image,
   const char
     *artifact;
 
+  const PixelPacket
+    *p;
+
   double
-    factor;
+    census;
 
   Image
     *edge_image;
@@ -157,9 +160,6 @@ static double GetEdgeBackgroundFactor(const Image *image,
   RectangleInfo
     edge_geometry;
 
-  const PixelPacket
-    *p;
-
   ssize_t
     y;
 
@@ -197,6 +197,8 @@ static double GetEdgeBackgroundFactor(const Image *image,
       break;
     }
   }
+  if (p == (const PixelPacket *) NULL)
+    return(0.0);
   GetMagickPixelPacket(image,&background);
   SetMagickPixelPacket(image,p,(IndexPacket *) NULL,&background);
   artifact=GetImageArtifact(image,"background");
@@ -213,7 +215,7 @@ static double GetEdgeBackgroundFactor(const Image *image,
   edge_image=CropImage(image,&edge_geometry,exception);
   if (edge_image == (Image *) NULL)
     return(0.0);
-  factor=0.0;
+  census=0.0;
   GetMagickPixelPacket(edge_image,&pixel);
   edge_view=AcquireVirtualCacheView(edge_image,exception);
   for (y=0; y < (ssize_t) edge_image->rows; y++)
@@ -228,24 +230,24 @@ static double GetEdgeBackgroundFactor(const Image *image,
     {
       SetMagickPixelPacket(edge_image,p,(IndexPacket *) NULL,&pixel);
       if (IsMagickColorSimilar(&pixel,&background) == MagickFalse)
-        factor++;
+        census++;
       p++;
     }
   }
-  factor/=((double) edge_image->columns*edge_image->rows);
+  census/=((double) edge_image->columns*edge_image->rows);
   edge_view=DestroyCacheView(edge_view);
   edge_image=DestroyImage(edge_image);
-  return(factor);
+  return(census);
 }
 
-static inline double GetMinEdgeBackgroundFactor(const EdgeInfo *edge)
+static inline double GetMinEdgeBackgroundCensus(const EdgeInfo *edge)
 {
   double
-    factor;
+    census;
 
-  factor=MagickMin(MagickMin(MagickMin(edge->left,edge->right),edge->top),
+  census=MagickMin(MagickMin(MagickMin(edge->left,edge->right),edge->top),
     edge->bottom);
-  return(factor);
+  return(census);
 }
 
 static RectangleInfo GetEdgeBoundingBox(const Image *image,
@@ -258,7 +260,7 @@ static RectangleInfo GetEdgeBoundingBox(const Image *image,
     *artifact;
 
   double
-    background_factor,
+    background_census,
     percent_background;
 
   EdgeInfo
@@ -276,22 +278,22 @@ static RectangleInfo GetEdgeBoundingBox(const Image *image,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   SetGeometry(image,&bounds);
   edge_image=CloneImage(image,0,0,MagickTrue,exception);
   if (edge_image == (Image *) NULL)
     return(bounds);
   (void) ParseAbsoluteGeometry("0x0+0+0",&edge_image->page);
-  memset(&vertex,0,sizeof(vertex));
+  (void) memset(&vertex,0,sizeof(vertex));
   edge_view=AcquireVirtualCacheView(edge_image,exception);
-  edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,WestGravity,
+  edge.left=GetEdgeBackgroundCensus(edge_image,edge_view,WestGravity,
     1,0,0,0,exception);
-  edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,EastGravity,
+  edge.right=GetEdgeBackgroundCensus(edge_image,edge_view,EastGravity,
     1,0,0,0,exception);
-  edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,NorthGravity,
+  edge.top=GetEdgeBackgroundCensus(edge_image,edge_view,NorthGravity,
     0,1,0,0,exception);
-  edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,SouthGravity,
+  edge.bottom=GetEdgeBackgroundCensus(edge_image,edge_view,SouthGravity,
     0,1,0,0,exception);
   percent_background=1.0;
   artifact=GetImageArtifact(edge_image,"trim:percent-background");
@@ -299,80 +301,80 @@ static RectangleInfo GetEdgeBoundingBox(const Image *image,
     percent_background=StringToDouble(artifact,(char **) NULL)/100.0;
   percent_background=MagickMin(MagickMax(1.0-percent_background,MagickEpsilon),
     1.0);
-  background_factor=GetMinEdgeBackgroundFactor(&edge);
-  for ( ; background_factor < percent_background;
-          background_factor=GetMinEdgeBackgroundFactor(&edge))
+  background_census=GetMinEdgeBackgroundCensus(&edge);
+  for ( ; background_census < percent_background;
+          background_census=GetMinEdgeBackgroundCensus(&edge))
   {
     if ((bounds.width == 0) || (bounds.height == 0))
       break;
-    if (fabs(edge.left-background_factor) < MagickEpsilon)
+    if (fabs(edge.left-background_census) < MagickEpsilon)
       {
         /*
           Trim left edge.
         */
         vertex.left++;
         bounds.width--;
-        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.left=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
           vertex.top,exception);
-        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.top=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
           vertex.top,exception);
-        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.bottom=GetEdgeBackgroundCensus(edge_image,edge_view,
           SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
           vertex.bottom,exception);
         continue;
       }
-    if (fabs(edge.right-background_factor) < MagickEpsilon)
+    if (fabs(edge.right-background_census) < MagickEpsilon)
       {
         /*
           Trim right edge.
         */
         vertex.right++;
         bounds.width--;
-        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.right=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
           vertex.top,exception);
-        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.top=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
           vertex.top,exception);
-        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.bottom=GetEdgeBackgroundCensus(edge_image,edge_view,
           SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
           vertex.bottom,exception);
         continue;
       }
-    if (fabs(edge.top-background_factor) < MagickEpsilon)
+    if (fabs(edge.top-background_census) < MagickEpsilon)
       {
         /*
           Trim top edge.
         */
         vertex.top++;
         bounds.height--;
-        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.left=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
           vertex.top,exception);
-        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.right=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
           vertex.top,exception);
-        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.top=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
           vertex.top,exception);
         continue;
       }
-    if (fabs(edge.bottom-background_factor) < MagickEpsilon)
+    if (fabs(edge.bottom-background_census) < MagickEpsilon)
       {
         /*
           Trim bottom edge.
         */
         vertex.bottom++;
         bounds.height--;
-        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.left=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
           vertex.top,exception);
-        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.right=GetEdgeBackgroundCensus(edge_image,edge_view,
           NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
           vertex.top,exception);
-        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+        edge.bottom=GetEdgeBackgroundCensus(edge_image,edge_view,
           SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
           vertex.bottom,exception);
         continue;
@@ -401,7 +403,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     status;
 
   MagickPixelPacket
-    target[3],
+    target[4],
     zero;
 
   RectangleInfo
@@ -415,13 +417,13 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   artifact=GetImageArtifact(image,"trim:percent-background");
   if (artifact != (const char *) NULL)
     return(GetEdgeBoundingBox(image,exception));
-  bounds.width=0;
-  bounds.height=0;
+  bounds.width=image->columns == 1 ? 1 : 0;
+  bounds.height=image->rows == 1 ? 1 : 0;
   bounds.x=(ssize_t) image->columns;
   bounds.y=(ssize_t) image->rows;
   GetMagickPixelPacket(image,&target[0]);
@@ -446,6 +448,12 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   if (p != (const PixelPacket *) NULL)
     SetMagickPixelPacket(image,p,GetCacheViewVirtualIndexQueue(image_view),
       &target[2]);
+  GetMagickPixelPacket(image,&target[3]);
+  p=GetCacheViewVirtualPixels(image_view,(ssize_t) image->columns-1,
+    (ssize_t) image->rows-1,1,1,exception);
+  if (p != (const PixelPacket *) NULL)
+    SetMagickPixelPacket(image,p,GetCacheViewVirtualIndexQueue(image_view),
+      &target[3]);
   status=MagickTrue;
   GetMagickPixelPacket(image,&zero);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -498,6 +506,13 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
       if ((y > (ssize_t) bounding_box.height) &&
           (IsMagickColorSimilar(&pixel,&target[2]) == MagickFalse))
         bounding_box.height=(size_t) y;
+      if ((x < (ssize_t) bounding_box.width) &&
+          (y > (ssize_t) bounding_box.height) &&
+          (IsMagickColorSimilar(&pixel,&target[3]) == MagickFalse))
+        {
+          bounding_box.width=(size_t) x;
+          bounding_box.height=(size_t) y;
+        }
       p++;
     }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -520,8 +535,8 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
       "GeometryDoesNotContainImage","`%s'",image->filename);
   else
     {
-      bounds.width-=(bounds.x-1);
-      bounds.height-=(bounds.y-1);
+      bounds.width-=(size_t) (bounds.x-1);
+      bounds.height-=(size_t) (bounds.y-1);
     }
   return(bounds);
 }
@@ -585,7 +600,7 @@ MagickExport size_t GetImageChannelDepth(const Image *image,
   assert(image != (Image *) NULL);
 
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   number_threads=(size_t) GetMagickResourceLimit(ThreadResource);
   current_depth=(size_t *) AcquireQuantumMemory(number_threads,
@@ -905,7 +920,7 @@ MagickExport ImageType GetImageType(const Image *image,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->colorspace == CMYKColorspace)
     {
@@ -976,7 +991,7 @@ MagickExport ImageType IdentifyImageGray(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->type == BilevelType) || (image->type == GrayscaleType) ||
       (image->type == GrayscaleMatteType))
@@ -1057,7 +1072,7 @@ MagickExport MagickBooleanType IdentifyImageMonochrome(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->type == BilevelType)
     return(MagickTrue);
@@ -1125,7 +1140,7 @@ MagickExport ImageType IdentifyImageType(const Image *image,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->colorspace == CMYKColorspace)
     {
@@ -1179,7 +1194,7 @@ MagickExport ImageType IdentifyImageType(const Image *image,
 %
 */
 MagickExport MagickBooleanType IsGrayImage(const Image *image,
-  ExceptionInfo *exception)
+  ExceptionInfo *magick_unused(exception))
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
@@ -1216,7 +1231,7 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
 %
 */
 MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
-  ExceptionInfo *exception)
+  ExceptionInfo *magick_unused(exception))
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
@@ -1272,7 +1287,7 @@ MagickExport MagickBooleanType IsOpaqueImage(const Image *image,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->matte == MagickFalse)
     return(MagickTrue);
@@ -1349,7 +1364,7 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
     y;
 
   assert(image != (Image *) NULL);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(image->signature == MagickCoreSignature);
   if (depth >= MAGICKCORE_QUANTUM_DEPTH)
@@ -1548,7 +1563,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     *quantize_info;
 
   assert(image != (Image *) NULL);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(image->signature == MagickCoreSignature);
   status=MagickTrue;
@@ -1561,8 +1576,10 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
   {
     case BilevelType:
     {
-      status=TransformImageColorspace(image,GRAYColorspace);
+      if (IsGrayColorspace(image->colorspace) == MagickFalse)
+        status=TransformImageColorspace(image,GRAYColorspace);
       (void) NormalizeImage(image);
+      (void) BilevelImage(image,(double) QuantumRange/2.0);
       quantize_info=AcquireQuantizeInfo(image_info);
       quantize_info->number_colors=2;
       quantize_info->colorspace=GRAYColorspace;
@@ -1573,20 +1590,23 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case GrayscaleType:
     {
-      status=TransformImageColorspace(image,GRAYColorspace);
+      if (IsGrayColorspace(image->colorspace) == MagickFalse)
+        status=TransformImageColorspace(image,GRAYColorspace);
       image->matte=MagickFalse;
       break;
     }
     case GrayscaleMatteType:
     {
-      status=TransformImageColorspace(image,GRAYColorspace);
+      if (IsGrayColorspace(image->colorspace) == MagickFalse)
+        status=TransformImageColorspace(image,GRAYColorspace);
       if (image->matte == MagickFalse)
         (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
       break;
     }
     case PaletteType:
     {
-      status=TransformImageColorspace(image,sRGBColorspace);
+      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+        status=TransformImageColorspace(image,sRGBColorspace);
       if ((image->storage_class == DirectClass) || (image->colors > 256))
         {
           quantize_info=AcquireQuantizeInfo(image_info);
@@ -1599,7 +1619,8 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case PaletteBilevelMatteType:
     {
-      status=TransformImageColorspace(image,sRGBColorspace);
+      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+        status=TransformImageColorspace(image,sRGBColorspace);
       if (image->matte == MagickFalse)
         (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
       (void) BilevelImageChannel(image,AlphaChannel,(double) QuantumRange/2.0);
@@ -1610,7 +1631,8 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case PaletteMatteType:
     {
-      status=TransformImageColorspace(image,sRGBColorspace);
+      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+        status=TransformImageColorspace(image,sRGBColorspace);
       if (image->matte == MagickFalse)
         (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
       quantize_info=AcquireQuantizeInfo(image_info);
@@ -1621,7 +1643,8 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case TrueColorType:
     {
-      status=TransformImageColorspace(image,sRGBColorspace);
+      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+        status=TransformImageColorspace(image,sRGBColorspace);
       if (image->storage_class != DirectClass)
         status=SetImageStorageClass(image,DirectClass);
       image->matte=MagickFalse;
@@ -1629,7 +1652,8 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case TrueColorMatteType:
     {
-      status=TransformImageColorspace(image,sRGBColorspace);
+      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+        status=TransformImageColorspace(image,sRGBColorspace);
       if (image->storage_class != DirectClass)
         status=SetImageStorageClass(image,DirectClass);
       if (image->matte == MagickFalse)
@@ -1638,7 +1662,8 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case ColorSeparationType:
     {
-      status=TransformImageColorspace(image,CMYKColorspace);
+      if (image->colorspace != CMYKColorspace)
+        status=TransformImageColorspace(image,CMYKColorspace);
       if (image->storage_class != DirectClass)
         status=SetImageStorageClass(image,DirectClass);
       image->matte=MagickFalse;
@@ -1646,7 +1671,8 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case ColorSeparationMatteType:
     {
-      status=TransformImageColorspace(image,CMYKColorspace);
+      if (image->colorspace != CMYKColorspace)
+        status=TransformImageColorspace(image,CMYKColorspace);
       if (image->storage_class != DirectClass)
         status=SetImageStorageClass(image,DirectClass);
       if (image->matte == MagickFalse)
