@@ -17,7 +17,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -77,6 +77,11 @@
 #include "magick/transform.h"
 #include "magick/utility.h"
 
+/*  
+  Define declarations.
+*/
+#define MaxReadRecursionDepth  100
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -146,10 +151,11 @@ MagickExport Image *ConstituteImage(const size_t columns,
     Allocate image structure.
   */
   assert(map != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",map);
   assert(pixels != (void *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",map);
   image=AcquireImage((ImageInfo *) NULL);
   if (image == (Image *) NULL)
     return((Image *) NULL);
@@ -279,10 +285,10 @@ MagickExport Image *PingImage(const ImageInfo *image_info,
 
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
+  assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(exception != (ExceptionInfo *) NULL);
   ping_info=CloneImageInfo(image_info);
   ping_info->ping=MagickTrue;
   image=ReadStream(ping_info,&PingStream,exception);
@@ -338,10 +344,10 @@ MagickExport Image *PingImages(const ImageInfo *image_info,
   */
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
+  assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(exception != (ExceptionInfo *) NULL);
   (void) InterpretImageFilename(image_info,(Image *) NULL,image_info->filename,
     (int) image_info->scene,filename);
   if (LocaleCompare(filename,image_info->filename) != 0)
@@ -468,10 +474,10 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
   assert(image_info->filename != (char *) NULL);
-  if (image_info->debug != MagickFalse)
+  assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(exception != (ExceptionInfo *) NULL);
   read_info=CloneImageInfo(image_info);
   (void) CopyMagickString(magick_filename,read_info->filename,MaxTextExtent);
   (void) SetImageInfo(read_info,0,exception);
@@ -680,7 +686,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       *source_date_epoch = (const char *) NULL;
 
     static MagickBooleanType
-      epoch_initalized = MagickFalse;
+      epoch_initialized = MagickFalse;
 
     next->taint=MagickFalse;
     GetPathComponent(magick_filename,MagickPath,magick_path);
@@ -793,15 +799,15 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
             else
               if (((flags & WidthValue) != 0) || ((flags & HeightValue) != 0))
                 {
-                  Image
-                    *size_image;
-
                   (void) ParseRegionGeometry(next,read_info->extract,&geometry,
                     exception);
-                  size_image=ResizeImage(next,geometry.width,geometry.height,
-                    next->filter,next->blur,exception);
-                  if (size_image != (Image *) NULL)
-                    ReplaceImageInList(&next,size_image);
+                  if ((geometry.width != 0) && (geometry.height != 0))
+                    {
+                      Image *resize_image=ResizeImage(next,geometry.width,
+                        geometry.height,next->filter,next->blur,exception);
+                      if (resize_image != (Image *) NULL)
+                        ReplaceImageInList(&next,resize_image);
+                    }
                 }
           }
       }
@@ -821,13 +827,15 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         next->iptc_profile.length=GetStringInfoLength(profile);
         next->iptc_profile.info=GetStringInfoDatum(profile);
       }
-    if (epoch_initalized == MagickFalse)
+    if (epoch_initialized == MagickFalse)
       {
         source_date_epoch=getenv("SOURCE_DATE_EPOCH");
-        epoch_initalized=MagickTrue;
+        epoch_initialized=MagickTrue;
       }
     if (source_date_epoch == (const char *) NULL)
       {
+        (void) FormatMagickTime(image->timestamp,MaxTextExtent,timestamp);
+        (void) SetImageProperty(next,"date:timestamp",timestamp);
         (void) FormatMagickTime((time_t) GetBlobProperties(next)->st_mtime,
           MaxTextExtent,timestamp);
         (void) SetImageProperty(next,"date:modify",timestamp);
@@ -851,8 +859,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
           if ((flags & LessValue) != 0)
             {
               if (next->delay < (size_t) floor(geometry_info.rho+0.5))
-                next->ticks_per_second=CastDoubleToLong(floor(
-                  geometry_info.sigma+0.5));
+                next->delay=(size_t) floor(geometry_info.rho+0.5);
             }
           else
             next->delay=(size_t) floor(geometry_info.rho+0.5);
@@ -920,10 +927,10 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
   */
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
+  assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(exception != (ExceptionInfo *) NULL);
   read_info=CloneImageInfo(image_info);
   *read_info->magick='\0';
   (void) InterpretImageFilename(read_info,(Image *) NULL,read_info->filename,
@@ -1048,7 +1055,7 @@ MagickExport Image *ReadInlineImage(const ImageInfo *image_info,
       */
       if (LocaleNCompare(++p,"x-",2) == 0)
         p+=2;
-      (void) strcpy(read_info->filename,"data.");
+      (void) CopyMagickString(read_info->filename,"data.",MagickPathExtent);
       q=read_info->filename+5;
       for (i=0; (*p != ';') && (*p != '\0') && (i < (MagickPathExtent-6)); i++)
         *q++=(*p++);
@@ -1123,10 +1130,10 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
   assert(image_info != (ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  if (image->debug != MagickFalse)
+  assert(image->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
-  assert(image->signature == MagickCoreSignature);
   exception=(&image->exception);
   sans_exception=AcquireExceptionInfo();
   write_info=CloneImageInfo(image_info);
@@ -1411,12 +1418,14 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(images != (Image *) NULL);
   assert(images->signature == MagickCoreSignature);
-  if (images->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   write_info=CloneImageInfo(image_info);
   *write_info->magick='\0';
   images=GetFirstImageInList(images);
+  if (images == (Image *) NULL)
+    return(MagickFalse);
   if (filename != (const char *) NULL)
     for (p=images; p != (Image *) NULL; p=GetNextImageInList(p))
       (void) CopyMagickString(p->filename,filename,MaxTextExtent);
