@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.  You may
@@ -18,6 +18,7 @@
 #ifndef MAGICKCORE_DELEGATE_PRIVATE_H
 #define MAGICKCORE_DELEGATE_PRIVATE_H
 
+#include "magick/locale_.h"
 #include "magick/string_.h"
 
 #if defined(MAGICKCORE_GS_DELEGATE)
@@ -107,6 +108,34 @@ static inline char *SanitizeDelegateString(const char *source)
   return(sanitize_source);
 }
 
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
+static inline void FormatSanitizedDelegateOption(char *string,
+  const size_t length,const char *windows_format,
+  const char *magick_unused(non_windows_format),const char *option)
+{
+  char
+    *sanitized_option;
+
+  magick_unreferenced(non_windows_format);
+  sanitized_option=SanitizeDelegateString(option);
+  (void) FormatLocaleString(string,length,windows_format,sanitized_option);
+  sanitized_option=DestroyString(sanitized_option);
+}
+#else
+static inline void FormatSanitizedDelegateOption(char *string,
+  const size_t length,const char *magick_unused(windows_format),
+  const char *non_windows_format,const char *option)
+{
+  char
+    *sanitized_option;
+
+  magick_unreferenced(windows_format);
+  sanitized_option=SanitizeDelegateString(option);
+  (void) FormatLocaleString(string,length,non_windows_format,sanitized_option);
+  sanitized_option=DestroyString(sanitized_option);
+}
+#endif
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
