@@ -18,7 +18,7 @@
 %                                August 2009                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -50,6 +50,7 @@
 #include "magick/histogram.h"
 #include "magick/image.h"
 #include "magick/list.h"
+#include "magick/locale_.h"
 #include "magick/memory_.h"
 #include "magick/monitor-private.h"
 #include "magick/pixel-private.h"
@@ -175,6 +176,35 @@ static inline size_t ColorToNodeId(const Image *image,
   return(id);
 }
 
+static inline MagickBooleanType IsMagickColorMatch(const MagickPixelPacket *p,
+  const MagickPixelPacket *q)
+{
+  MagickRealType
+    alpha,
+    beta;
+
+  alpha=p->matte == MagickFalse ? (MagickRealType) OpaqueOpacity : p->opacity;
+  beta=q->matte == MagickFalse ? (MagickRealType) OpaqueOpacity : q->opacity;
+  if (AbsolutePixelValue(alpha-beta) >= MagickEpsilon)
+    return(MagickFalse);
+  if ((AbsolutePixelValue(alpha-(MagickRealType) TransparentOpacity) < MagickEpsilon) ||
+      (AbsolutePixelValue(beta-(MagickRealType) TransparentOpacity) < MagickEpsilon))
+    return(MagickTrue);  /* no color component if pixel is transparent */
+  if (AbsolutePixelValue(p->red-q->red) >= MagickEpsilon)
+    return(MagickFalse);
+  if (AbsolutePixelValue(p->green-q->green) >= MagickEpsilon)
+    return(MagickFalse);
+  if (AbsolutePixelValue(p->blue-q->blue) >= MagickEpsilon)
+    return(MagickFalse);
+  if (p->colorspace == CMYKColorspace)
+    {
+      if (AbsolutePixelValue(p->index-q->index) >= MagickEpsilon)
+        return(MagickFalse);
+    }
+  return(MagickTrue);
+}
+
+
 static CubeInfo *ClassifyImageColors(const Image *image,
   ExceptionInfo *exception)
 {
@@ -196,18 +226,18 @@ static CubeInfo *ClassifyImageColors(const Image *image,
   NodeInfo
     *node_info;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register size_t
+  size_t
     id,
     index,
     level;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
@@ -219,7 +249,7 @@ static CubeInfo *ClassifyImageColors(const Image *image,
   */
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   cube_info=GetCubeInfo();
   if (cube_info == (CubeInfo *) NULL)
@@ -266,7 +296,7 @@ static CubeInfo *ClassifyImageColors(const Image *image,
       {
         SetMagickPixelPacket(image,&node_info->list[i].pixel,
           &node_info->list[i].index,&target);
-        if (IsMagickColorEqual(&pixel,&target) != MagickFalse)
+        if (IsMagickColorMatch(&pixel,&target) != MagickFalse)
           break;
       }
       if (i < (ssize_t) node_info->number_unique)
@@ -345,7 +375,7 @@ static CubeInfo *ClassifyImageColors(const Image *image,
 static void DefineImageHistogram(const Image *image,NodeInfo *node_info,
   ColorPacket **histogram)
 {
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -360,7 +390,7 @@ static void DefineImageHistogram(const Image *image,NodeInfo *node_info,
       DefineImageHistogram(image,node_info->child[i],histogram);
   if (node_info->level == (MaxTreeDepth-1))
     {
-      register ColorPacket
+      ColorPacket
         *p;
 
       p=node_info->list;
@@ -401,7 +431,7 @@ static void DefineImageHistogram(const Image *image,NodeInfo *node_info,
 */
 static CubeInfo *DestroyCubeInfo(const Image *image,CubeInfo *cube_info)
 {
-  register Nodes
+  Nodes
     *nodes;
 
   /*
@@ -446,7 +476,7 @@ static CubeInfo *DestroyCubeInfo(const Image *image,CubeInfo *cube_info)
 */
 static void DestroyColorCube(const Image *image,NodeInfo *node_info)
 {
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -561,8 +591,8 @@ MagickExport ColorPacket *GetImageHistogram(const Image *image,
           root=histogram;
           DefineImageHistogram(image,cube_info->root,&root);
         }
+      cube_info=DestroyCubeInfo(image,cube_info);
     }
-  cube_info=DestroyCubeInfo(image,cube_info);
   return(histogram);
 }
 
@@ -659,19 +689,19 @@ static MagickBooleanType CheckImageColors(const Image *image,
     pixel,
     target;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register NodeInfo
+  NodeInfo
     *node_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -703,6 +733,8 @@ static MagickBooleanType CheckImageColors(const Image *image,
     if (p == (const PixelPacket *) NULL)
       break;
     indexes=GetCacheViewVirtualIndexQueue(image_view);
+    if (indexes == (const IndexPacket *) NULL)
+      break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
       /*
@@ -734,7 +766,7 @@ static MagickBooleanType CheckImageColors(const Image *image,
       {
         SetMagickPixelPacket(image,&node_info->list[i].pixel,
           &node_info->list[i].index,&target);
-        if (IsMagickColorEqual(&pixel,&target) != MagickFalse)
+        if (IsMagickColorMatch(&pixel,&target) != MagickFalse)
           break;
       }
       if (i < (ssize_t) node_info->number_unique)
@@ -744,8 +776,8 @@ static MagickBooleanType CheckImageColors(const Image *image,
           /*
             Add this unique color to the color list.
           */
-          if (node_info->number_unique == 0)
-            node_info->list=(ColorPacket *) AcquireMagickMemory(
+          if (node_info->list == (ColorPacket *) NULL)
+            node_info->list=(ColorPacket *) AcquireQuantumMemory(1,
               sizeof(*node_info->list));
           else
             node_info->list=(ColorPacket *) ResizeQuantumMemory(node_info->list,
@@ -782,7 +814,7 @@ MagickExport MagickBooleanType IdentifyPaletteImage(const Image *image,
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   return(CheckImageColors(image,exception,256));
 }
@@ -828,19 +860,19 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
     pixel,
     target;
 
-  register const IndexPacket
+  const IndexPacket
     *indexes;
 
-  register const PixelPacket
+  const PixelPacket
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register NodeInfo
+  NodeInfo
     *node_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -853,7 +885,7 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->storage_class == PseudoClass) &&
       (image->colors <= MaximumUniqueColors))
@@ -910,7 +942,7 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
       {
         SetMagickPixelPacket(image,&node_info->list[i].pixel,
           &node_info->list[i].index,&target);
-        if (IsMagickColorEqual(&pixel,&target) != MagickFalse)
+        if (IsMagickColorMatch(&pixel,&target) != MagickFalse)
           break;
       }
       if (i < (ssize_t) node_info->number_unique)
@@ -921,7 +953,7 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
             Add this unique color to the color list.
           */
           if (node_info->number_unique == 0)
-            node_info->list=(ColorPacket *) AcquireMagickMemory(
+            node_info->list=(ColorPacket *) AcquireQuantumMemory(1,
               sizeof(*node_info->list));
           else
             node_info->list=(ColorPacket *) ResizeQuantumMemory(node_info->list,
@@ -980,11 +1012,12 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
 %
 */
 MagickExport MagickBooleanType IsPaletteImage(const Image *image,
-  ExceptionInfo *exception)
+  ExceptionInfo *magick_unused(exception))
 {
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  magick_unreferenced(exception);
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->storage_class != PseudoClass)
     return(MagickFalse);
@@ -1010,7 +1043,7 @@ MagickExport MagickBooleanType IsPaletteImage(const Image *image,
 %  If the adjustment values are both zero this function is equivalent to a
 %  perfect normalization (or autolevel) of the image.
 %
-%  Each channel is stretched independantally of each other (producing color
+%  Each channel is stretched independently of each other (producing color
 %  distortion) unless the special 'SyncChannels' flag is also provided in the
 %  channels setting. If this flag is present the minimum and maximum point
 %  will be extracted from all the given channels, and those channels will be
@@ -1156,12 +1189,12 @@ static int HistogramCompare(const void *x,const void *y)
   color_1=(const ColorPacket *) x;
   color_2=(const ColorPacket *) y;
   if (color_2->pixel.red != color_1->pixel.red)
-    return((int) color_1->pixel.red-(int) color_2->pixel.red);
+    return((int) ((ssize_t) color_1->pixel.red-(ssize_t) color_2->pixel.red));
   if (color_2->pixel.green != color_1->pixel.green)
-    return((int) color_1->pixel.green-(int) color_2->pixel.green);
+    return((int) ((ssize_t) color_1->pixel.green-(ssize_t) color_2->pixel.green));
   if (color_2->pixel.blue != color_1->pixel.blue)
-    return((int) color_1->pixel.blue-(int) color_2->pixel.blue);
-  return((int) color_2->count-(int) color_1->count);
+    return((int) ((ssize_t) color_1->pixel.blue-(ssize_t) color_2->pixel.blue));
+  return((int) ((ssize_t) color_2->count-(ssize_t) color_1->count));
 }
 
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -1175,6 +1208,7 @@ MagickExport size_t GetNumberColors(const Image *image,FILE *file,
 
   char
     color[MaxTextExtent],
+    count[MaxTextExtent],
     hex[MaxTextExtent],
     tuple[MaxTextExtent];
 
@@ -1187,10 +1221,10 @@ MagickExport size_t GetNumberColors(const Image *image,FILE *file,
   MagickPixelPacket
     pixel;
 
-  register ColorPacket
+  ColorPacket
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1204,8 +1238,10 @@ MagickExport size_t GetNumberColors(const Image *image,FILE *file,
 
       cube_info=ClassifyImageColors(image,exception);
       if (cube_info != (CubeInfo *) NULL)
-        number_colors=cube_info->colors;
-      cube_info=DestroyCubeInfo(image,cube_info);
+        {
+          number_colors=cube_info->colors;
+          cube_info=DestroyCubeInfo(image,cube_info);
+        }
       return(number_colors);
     }
   histogram=GetImageHistogram(image,&number_colors,exception);
@@ -1220,27 +1256,27 @@ MagickExport size_t GetNumberColors(const Image *image,FILE *file,
   {
     SetMagickPixelPacket(image,&p->pixel,&p->index,&pixel);
     (void) CopyMagickString(tuple,"(",MaxTextExtent);
-    ConcatenateColorComponent(&pixel,RedChannel,X11Compliance,tuple);
+    ConcatenateColorComponent(&pixel,RedChannel,NoCompliance,tuple);
     (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
-    ConcatenateColorComponent(&pixel,GreenChannel,X11Compliance,tuple);
+    ConcatenateColorComponent(&pixel,GreenChannel,NoCompliance,tuple);
     (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
-    ConcatenateColorComponent(&pixel,BlueChannel,X11Compliance,tuple);
+    ConcatenateColorComponent(&pixel,BlueChannel,NoCompliance,tuple);
     if (pixel.colorspace == CMYKColorspace)
       {
         (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
-        ConcatenateColorComponent(&pixel,IndexChannel,X11Compliance,tuple);
+        ConcatenateColorComponent(&pixel,IndexChannel,NoCompliance,tuple);
       }
     if (pixel.matte != MagickFalse)
       {
         (void) ConcatenateMagickString(tuple,",",MaxTextExtent);
-        ConcatenateColorComponent(&pixel,OpacityChannel,X11Compliance,tuple);
+        ConcatenateColorComponent(&pixel,OpacityChannel,NoCompliance,tuple);
       }
     (void) ConcatenateMagickString(tuple,")",MaxTextExtent);
     (void) QueryMagickColorname(image,&pixel,SVGCompliance,color,exception);
     GetColorTuple(&pixel,MagickTrue,hex);
-    (void) FormatLocaleFile(file,"%10.20g",(double) ((MagickOffsetType)
-      p->count));
-    (void) FormatLocaleFile(file,": %s %s %s\n",tuple,hex,color);
+    (void) FormatLocaleString(count,MagickPathExtent,"%10.20g:",(double)
+      ((MagickOffsetType) p->count));
+    (void) FormatLocaleFile(file,"    %s %s %s %s\n",count,tuple,hex,color);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
@@ -1293,7 +1329,7 @@ static void UniqueColorsToImage(Image *unique_image,CacheView *unique_view,
   MagickBooleanType
     status;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1309,13 +1345,13 @@ static void UniqueColorsToImage(Image *unique_image,CacheView *unique_view,
         node_info->child[i],exception);
   if (node_info->level == (MaxTreeDepth-1))
     {
-      register ColorPacket
+      ColorPacket
         *p;
 
-      register IndexPacket
+      IndexPacket
         *magick_restrict indexes;
 
-      register PixelPacket
+      PixelPacket
         *magick_restrict q;
 
       status=MagickTrue;
