@@ -1,8 +1,8 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
   
     https://imagemagick.org/script/license.php
@@ -289,7 +289,7 @@ struct _Image
   MagickBooleanType
     debug;            /* debug output attribute */
 
-  volatile ssize_t
+  ssize_t
     reference_count;
 
   SemaphoreInfo
@@ -396,7 +396,7 @@ struct _ImageInfo
     quality;
 
   char
-    *sampling_factor,
+    *sampling_factor,  /* Chroma subsampling ratio string */
     *server_name,
     *font,
     *texture,
