@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.  You may
@@ -43,6 +43,23 @@ extern "C" {
 #define MagickAssumeAligned(address)  (address)
 #endif
 
+static inline size_t OverAllocateMemory(const size_t length)
+{
+  size_t
+    extent;
+
+  /*
+    Over allocate memory, typically used when concatenating strings.
+  */
+  extent=length;
+  if (extent < 131072)
+    for (extent=256; extent < length; extent*=2);
+  return(extent);
+}
+
+extern MagickPrivate MagickBooleanType
+  ShredMagickMemory(void *,const size_t);
+
 extern MagickPrivate void
   ResetMaxMemoryRequest(void),
   ResetVirtualAnonymousMemory(void);
