@@ -17,7 +17,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -301,8 +301,9 @@ MagickExport const ModuleInfo **GetModuleInfoList(const char *pattern,
     Allocate module list.
   */
   assert(pattern != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   assert(number_modules != (size_t *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",pattern);
   *number_modules=0;
   p=GetModuleInfo("*",exception);
   if (p == (const ModuleInfo *) NULL)
@@ -531,9 +532,10 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     *module_path;
 
   assert(filename != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   if (strchr(filename,'/') != (char *) NULL)
     return(MagickFalse);
   (void) CopyMagickString(path,filename,MaxTextExtent);
@@ -635,7 +637,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
     {
       const char
-        *registery_key;
+        *registry_key;
 
       unsigned char
         *key_value;
@@ -648,20 +650,20 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
         case MagickImageCoderModule:
         default:
         {
-          registery_key="CoderModulesPath";
+          registry_key="CoderModulesPath";
           break;
         }
         case MagickImageFilterModule:
         {
-          registery_key="FilterModulesPath";
+          registry_key="FilterModulesPath";
           break;
         }
       }
-      key_value=NTRegistryKeyLookup(registery_key);
+      key_value=NTRegistryKeyLookup(registry_key);
       if (key_value == (unsigned char *) NULL)
         {
           ThrowMagickException(exception,GetMagickModule(),ConfigureError,
-            "RegistryKeyLookupFailed","`%s'",registery_key);
+            "RegistryKeyLookupFailed","`%s'",registry_key);
           return(MagickFalse);
         }
       (void) FormatLocaleString(path,MaxTextExtent,"%s%s%s",(char *) key_value,
@@ -981,7 +983,7 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
   */
   assert(images != (Image **) NULL);
   assert((*images)->signature == MagickCoreSignature);
-  if ((*images)->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
   rights=ReadPolicyRights;
@@ -1042,11 +1044,11 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
       size_t
         signature;
 
-      if ((*images)->debug != MagickFalse)
+      if (IsEventLogging() != MagickFalse)
         (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
           "Invoking \"%s\" dynamic image filter",tag);
       signature=image_filter(images,argc,argv,exception);
-      if ((*images)->debug != MagickFalse)
+      if (IsEventLogging() != MagickFalse)
         (void) LogMagickEvent(ModuleEvent,GetMagickModule(),"\"%s\" completes",
           tag);
       if (signature != MagickImageFilterSignature)
@@ -1279,18 +1281,19 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   module_info=(ModuleInfo *) GetModuleInfo(module,exception);
   if (module_info != (ModuleInfo *) NULL)
     return(MagickTrue);
-  rights=ReadPolicyRights;
-  if (IsRightsAuthorized(ModulePolicyDomain,rights,module) == MagickFalse)
+  (void) CopyMagickString(module_name,module,MaxTextExtent);
+  p=GetCoderInfo(module,exception);
+  if (p != (CoderInfo *) NULL)
+    (void) CopyMagickString(module_name,p->name,MaxTextExtent);
+  LocaleUpper(module_name);
+  rights=(PolicyRights) (ReadPolicyRights | WritePolicyRights);
+  if (IsRightsAuthorized(ModulePolicyDomain,rights,module_name) == MagickFalse)
     {
       errno=EPERM;
       (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
         "NotAuthorized","`%s'",module);
       return(MagickFalse);
     }
-  (void) CopyMagickString(module_name,module,MaxTextExtent);
-  p=GetCoderInfo(module,exception);
-  if (p != (CoderInfo *) NULL)
-    (void) CopyMagickString(module_name,p->name,MaxTextExtent);
   if (GetValueFromSplayTree(module_list,module_name) != (void *) NULL)
     return(MagickTrue);  /* module already opened, return */
   /*
@@ -1454,7 +1457,8 @@ static const ModuleInfo *RegisterModule(const ModuleInfo *module_info,
 
   assert(module_info != (ModuleInfo *) NULL);
   assert(module_info->signature == MagickCoreSignature);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",module_info->tag);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",module_info->tag);
   if (module_list == (SplayTreeInfo *) NULL)
     return((const ModuleInfo *) NULL);
   status=AddValueToSplayTree(module_list,module_info->tag,module_info);
@@ -1492,8 +1496,9 @@ static const ModuleInfo *RegisterModule(const ModuleInfo *module_info,
 static void TagToCoderModuleName(const char *tag,char *name)
 {
   assert(tag != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
   assert(name != (char *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
 #if defined(MAGICKCORE_LTDL_DELEGATE)
   (void) FormatLocaleString(name,MaxTextExtent,"%s.la",tag);
   (void) LocaleLower(name);
@@ -1541,8 +1546,9 @@ static void TagToCoderModuleName(const char *tag,char *name)
 static void TagToFilterModuleName(const char *tag,char *name)
 {
   assert(tag != (char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
   assert(name != (char *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
 #if !defined(MAGICKCORE_LTDL_DELEGATE)
   (void) FormatLocaleString(name,MaxTextExtent,"%s.dll",tag);
 #else
@@ -1584,9 +1590,10 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
     name[MaxTextExtent];
 
   assert(tag != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
   assert(format != (const char *) NULL);
   assert(module != (char *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",tag);
   (void) CopyMagickString(name,tag,MaxTextExtent);
   LocaleUpper(name);
 #if !defined(MAGICKCORE_NAMESPACE_PREFIX)
@@ -1636,8 +1643,9 @@ static MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
     Locate and execute UnregisterFORMATImage module.
   */
   assert(module_info != (const ModuleInfo *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",module_info->tag);
   assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",module_info->tag);
   if (module_info->unregister_module == NULL)
     return(MagickTrue);
   module_info->unregister_module();
@@ -1672,7 +1680,7 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
 
   assert(image != (Image **) NULL);
   assert((*image)->signature == MagickCoreSignature);
-  if ((*image)->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",(*image)->filename);
   rights=ReadPolicyRights;
   if (IsRightsAuthorized(FilterPolicyDomain,rights,tag) == MagickFalse)
@@ -1704,12 +1712,12 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
           signature;
 
         if ((*image)->debug != MagickFalse)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+          (void) LogMagickEvent(TransformEvent,GetMagickModule(),
             "Invoking \"%s\" static image filter",tag);
         signature=image_filter(image,argc,argv,exception);
         if ((*image)->debug != MagickFalse)
-          (void) LogMagickEvent(CoderEvent,GetMagickModule(),"\"%s\" completes",
-            tag);
+          (void) LogMagickEvent(TransformEvent,GetMagickModule(),
+            "\"%s\" completes",tag);
         if (signature != MagickImageFilterSignature)
           {
             (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
