@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -58,6 +58,7 @@
 #include "magick/image.h"
 #include "magick/image-private.h"
 #include "magick/layer.h"
+#include "magick/locale_.h"
 #include "magick/mime-private.h"
 #include "magick/memory_.h"
 #include "magick/memory-private.h"
@@ -89,11 +90,10 @@
 #include "CLPerfMarker.h"
 #endif
 
-
 #if defined(MAGICKCORE_OPENCL_SUPPORT)
 
-#ifdef MAGICKCORE_HAVE_OPENCL_CL_H
-#define MAGICKCORE_OPENCL_MACOSX  1
+#if defined(MAGICKCORE_LTDL_DELEGATE)
+#include "ltdl.h"
 #endif
 
 #define NUM_CL_RAND_GENERATORS 1024  /* number of random number generators running in parallel */
@@ -254,9 +254,13 @@ void DumpProfileData()
   for (i = 0; i < KERNEL_COUNT; ++i) {
     char buf[4096];
     char indent[160];
-    strcpy(indent, "                              ");
+    (void) CopyMagickString(indent,"                              ",
+      sizeof(indent);
     strncpy(indent, kernelNames[i], min(strlen(kernelNames[i]), strlen(indent) - 1));
-    sprintf(buf, "%s%d\t(%d calls)   \t%d -> %d", indent, profileRecords[i].count > 0 ? (profileRecords[i].total / profileRecords[i].count) : 0, profileRecords[i].count, profileRecords[i].min, profileRecords[i].max);
+    (void) FormatLocaleString(buf,sizeof(buf),"%s%d\t(%d calls)   \t%d -> %d",
+      indent, profileRecords[i].count > 0 ? (profileRecords[i].total /
+      profileRecords[i].count) : 0, profileRecords[i].count,
+      profileRecords[i].min, profileRecords[i].max);
     /*
       printf("%s%d\t(%d calls)   \t%d -> %d\n", indent, profileRecords[i].count > 0 ? (profileRecords[i].total / profileRecords[i].count) : 0, profileRecords[i].count, profileRecords[i].min, profileRecords[i].max);
     */
@@ -395,7 +399,7 @@ SemaphoreInfo* OpenCLLibLock;
 
 static MagickBooleanType bindOpenCLFunctions(void* library)
 {
-#ifdef MAGICKCORE_OPENCL_MACOSX
+#ifdef MAGICKCORE_HAVE_OPENCL_CL_H
 #define BIND(X) OpenCLLib->X= &X;
 #else
 #define BIND(X)\
@@ -793,7 +797,7 @@ MagickExport
       *((char **) data),NULL);
     status = MagickTrue;
     break;
-  
+
   default:
     goto cleanup;
   };
@@ -867,10 +871,12 @@ static void saveBinaryCLProgram(MagickCLEnv clEnv,MagickOpenCLProgram prog,
   char
     *filename;
 
-  cl_uint
-    num_devices,
+  cl_int
     status;
 
+  cl_uint
+    num_devices;
+
   size_t
     i,
     size,
@@ -882,7 +888,7 @@ static void saveBinaryCLProgram(MagickCLEnv clEnv,MagickOpenCLProgram prog,
   if (status != CL_SUCCESS)
     return;
   size=num_devices*sizeof(*program_sizes);
-  program_sizes=(size_t*) AcquireMagickMemory(size);
+  program_sizes=(size_t*) AcquireQuantumMemory(1,size);
   if (program_sizes == (size_t*) NULL)
     return;
   status=clEnv->library->clGetProgramInfo(clEnv->programs[prog],
@@ -896,7 +902,7 @@ static void saveBinaryCLProgram(MagickCLEnv clEnv,MagickOpenCLProgram prog,
         **binary_program;
 
       binary_program_size=num_devices*sizeof(*binary_program);
-      binary_program=(unsigned char **) AcquireMagickMemory(
+      binary_program=(unsigned char **) AcquireQuantumMemory(1,
         binary_program_size);
       if (binary_program == (unsigned char **) NULL)
         {
@@ -1001,7 +1007,7 @@ cleanup:
   if (fileHandle != NULL)
     fclose(fileHandle);
   if (binaryFileName != NULL)
-    free(binaryFileName);
+    RelinquishMagickMemory(binaryFileName);
   if (binaryProgram != NULL)
     RelinquishMagickMemory(binaryProgram);
 
@@ -1097,8 +1103,9 @@ static MagickBooleanType CompileOpenCLKernels(MagickCLEnv clEnv, ExceptionInfo*
   optionsSignature = stringSignature(options);
 
   /* get all the OpenCL program strings here */
-  accelerateKernelsBuffer = (char*) AcquireMagickMemory(strlen(accelerateKernels)+strlen(accelerateKernels2)+1);
-  sprintf(accelerateKernelsBuffer,"%s%s",accelerateKernels,accelerateKernels2);
+  accelerateKernelsBuffer = (char*) AcquireQuantumMemory(1,strlen(accelerateKernels)+strlen(accelerateKernels2)+1);
+  FormatLocaleString(accelerateKernelsBuffer,strlen(accelerateKernels)+
+    strlen(accelerateKernels2)+1,"%s%s",accelerateKernels,accelerateKernels2);
   MagickOpenCLProgramStrings[MAGICK_OPENCL_ACCELERATE] = accelerateKernelsBuffer;
 
   for (i = 0; i < MAGICK_OPENCL_NUM_PROGRAMS; i++)
@@ -1208,25 +1215,19 @@ static MagickBooleanType InitOpenCLPlatformDevice(MagickCLEnv clEnv, ExceptionIn
 
   /* check if there's an environment variable overriding the device selection */
   MAGICK_OCL_DEVICE = getenv("MAGICK_OCL_DEVICE");
-  if (MAGICK_OCL_DEVICE != NULL)
-  {
-    if (strcmp(MAGICK_OCL_DEVICE, "CPU") == 0)
+  if (MAGICK_OCL_DEVICE == (char *) NULL)
+    return(MagickFalse);
+  if (strcmp(MAGICK_OCL_DEVICE, "CPU") == 0)
+    clEnv->deviceType = CL_DEVICE_TYPE_CPU;
+  else if (strcmp(MAGICK_OCL_DEVICE, "GPU") == 0)
+    clEnv->deviceType = CL_DEVICE_TYPE_GPU;
+  else if (IsStringTrue(MAGICK_OCL_DEVICE) != MagickFalse)
     {
-      clEnv->deviceType = CL_DEVICE_TYPE_CPU;
+      if (clEnv->deviceType == 0)
+        clEnv->deviceType = CL_DEVICE_TYPE_ALL;
     }
-    else if (strcmp(MAGICK_OCL_DEVICE, "GPU") == 0)
-    {
-      clEnv->deviceType = CL_DEVICE_TYPE_GPU;
-    }
-    else if (strcmp(MAGICK_OCL_DEVICE, "OFF") == 0)
-    {
-      /* OpenCL disabled */
-      goto cleanup;
-    }
-  }
-  else if (clEnv->deviceType == 0) {
-    clEnv->deviceType = CL_DEVICE_TYPE_ALL;
-  }
+  else
+    return(MagickFalse);
 
   if (clEnv->device != NULL)
   {
@@ -1240,7 +1241,7 @@ static MagickBooleanType InitOpenCLPlatformDevice(MagickCLEnv clEnv, ExceptionIn
   else if (clEnv->platform != NULL)
   {
     numPlatforms = 1;
-    platforms = (cl_platform_id *) AcquireMagickMemory(numPlatforms * sizeof(cl_platform_id));
+    platforms = (cl_platform_id *) AcquireQuantumMemory(1,numPlatforms * sizeof(cl_platform_id));
     if (platforms == (cl_platform_id *) NULL)
     {
       (void) ThrowMagickException(exception, GetMagickModule(), ResourceLimitError,
@@ -1267,7 +1268,7 @@ static MagickBooleanType InitOpenCLPlatformDevice(MagickCLEnv clEnv, ExceptionIn
       goto cleanup;
     }
 
-    platforms = (cl_platform_id *) AcquireMagickMemory(numPlatforms * sizeof(cl_platform_id));
+    platforms = (cl_platform_id *) AcquireQuantumMemory(1,numPlatforms * sizeof(cl_platform_id));
     if (platforms == (cl_platform_id *) NULL)
     {
       (void) ThrowMagickException(exception, GetMagickModule(), ResourceLimitError,
@@ -1472,7 +1473,7 @@ MagickExport
 MagickBooleanType InitOpenCLEnv(MagickCLEnv clEnv, ExceptionInfo* exception) {
   MagickBooleanType status = MagickFalse;
 
-  if (clEnv == NULL)
+  if ((clEnv == NULL) || (getenv("MAGICK_OCL_DEVICE") == (const char *) NULL))
     return MagickFalse;
 
 #ifdef MAGICKCORE_CLPERFMARKER
@@ -1481,8 +1482,7 @@ MagickBooleanType InitOpenCLEnv(MagickCLEnv clEnv, ExceptionInfo* exception) {
 
   LockSemaphoreInfo(clEnv->lock);
   if (clEnv->OpenCLInitialized == MagickFalse) {
-    if (clEnv->device==NULL
-        && clEnv->OpenCLDisabled == MagickFalse)
+    if (clEnv->device==NULL && clEnv->OpenCLDisabled == MagickFalse)
       status = autoSelectDevice(clEnv, exception);
     else
       status = InitOpenCLEnvInternal(clEnv, exception);
@@ -1771,8 +1771,8 @@ typedef ds_status (*ds_score_release)(void* score);
 static ds_status releaseDeviceResource(ds_device* device, ds_score_release sr) {
   ds_status status = DS_SUCCESS;
   if (device) {
-    if (device->oclDeviceName)      free(device->oclDeviceName);
-    if (device->oclDriverVersion)   free(device->oclDriverVersion);
+    if (device->oclDeviceName)      RelinquishMagickMemory(device->oclDeviceName);
+    if (device->oclDriverVersion)   RelinquishMagickMemory(device->oclDriverVersion);
     if (device->score)              status = sr(device->score);
   }
   return status;
@@ -1788,9 +1788,9 @@ static ds_status releaseDSProfile(ds_profile* profile, ds_score_release sr) {
         if (status != DS_SUCCESS)
           break;
       }
-      free(profile->devices);
+      RelinquishMagickMemory(profile->devices);
     }
-    free(profile);
+    RelinquishMagickMemory(profile);
   }
   return status;
 }
@@ -1809,7 +1809,7 @@ static ds_status initDSProfile(ds_profile** p, const char* version) {
   if (p == NULL)
     return DS_INVALID_PROFILE;
 
-  profile = (ds_profile*)malloc(sizeof(ds_profile));
+  profile = (ds_profile*) AcquireMagickMemory(sizeof(ds_profile));
   if (profile == NULL)
     return DS_MEMORY_ERROR;
 
@@ -1817,7 +1817,7 @@ static ds_status initDSProfile(ds_profile** p, const char* version) {
 
   OpenCLLib->clGetPlatformIDs(0, NULL, &numPlatforms);
   if (numPlatforms > 0) {
-    platforms = (cl_platform_id*)malloc(numPlatforms*sizeof(cl_platform_id));
+    platforms = (cl_platform_id*) AcquireQuantumMemory(numPlatforms,sizeof(cl_platform_id));
     if (platforms == NULL) {
       status = DS_MEMORY_ERROR;
       goto cleanup;
@@ -1832,7 +1832,7 @@ static ds_status initDSProfile(ds_profile** p, const char* version) {
 
   profile->numDevices = numDevices+1;     /* +1 to numDevices to include the native CPU */
 
-  profile->devices = (ds_device*)malloc(profile->numDevices*sizeof(ds_device));
+  profile->devices = (ds_device*) AcquireQuantumMemory(profile->numDevices,sizeof(ds_device));
   if (profile->devices == NULL) {
     profile->numDevices = 0;
     status = DS_MEMORY_ERROR;
@@ -1841,7 +1841,7 @@ static ds_status initDSProfile(ds_profile** p, const char* version) {
   memset(profile->devices, 0, profile->numDevices*sizeof(ds_device));
 
   if (numDevices > 0) {
-    devices = (cl_device_id*)malloc(numDevices*sizeof(cl_device_id));
+    devices = (cl_device_id*) AcquireQuantumMemory(numDevices,sizeof(cl_device_id));
     if (devices == NULL) {
       status = DS_MEMORY_ERROR;
       goto cleanup;
@@ -1874,13 +1874,13 @@ static ds_status initDSProfile(ds_profile** p, const char* version) {
 
           OpenCLLib->clGetDeviceInfo(profile->devices[next].oclDeviceID, CL_DEVICE_NAME
             , 0, NULL, &length);
-          profile->devices[next].oclDeviceName = (char*)malloc(sizeof(char)*length);
+          profile->devices[next].oclDeviceName = (char*) AcquireQuantumMemory(length,sizeof(char));
           OpenCLLib->clGetDeviceInfo(profile->devices[next].oclDeviceID, CL_DEVICE_NAME
             , length, profile->devices[next].oclDeviceName, NULL);
 
           OpenCLLib->clGetDeviceInfo(profile->devices[next].oclDeviceID, CL_DRIVER_VERSION
             , 0, NULL, &length);
-          profile->devices[next].oclDriverVersion = (char*)malloc(sizeof(char)*length);
+          profile->devices[next].oclDriverVersion = (char*) AcquireQuantumMemory(length,sizeof(char));
           OpenCLLib->clGetDeviceInfo(profile->devices[next].oclDeviceID, CL_DRIVER_VERSION
             , length, profile->devices[next].oclDriverVersion, NULL);
 
@@ -1901,16 +1901,16 @@ static ds_status initDSProfile(ds_profile** p, const char* version) {
   profile->version = version;
 
 cleanup:
-  if (platforms)  free(platforms);
-  if (devices)    free(devices);
+  if (platforms)  RelinquishMagickMemory(platforms);
+  if (devices)    RelinquishMagickMemory(devices);
   if (status == DS_SUCCESS) {
     *p = profile;
   }
   else {
     if (profile) {
       if (profile->devices)
-        free(profile->devices);
-      free(profile);
+        RelinquishMagickMemory(profile->devices);
+      RelinquishMagickMemory(profile);
     }
   }
   return status;
@@ -2044,12 +2044,14 @@ static ds_status writeProfileToFile(ds_profile* profile, ds_score_serializer ser
           fwrite(DS_TAG_DEVICE_DRIVER_VERSION_END, sizeof(char), strlen(DS_TAG_DEVICE_DRIVER_VERSION_END), profileFile);
 
           fwrite(DS_TAG_DEVICE_MAX_COMPUTE_UNITS, sizeof(char), strlen(DS_TAG_DEVICE_MAX_COMPUTE_UNITS), profileFile);
-          sprintf(tmp,"%d",profile->devices[i].oclMaxComputeUnits);
+          (void) FormatLocaleString(tmp,sizeof(tmp),"%d",
+            profile->devices[i].oclMaxComputeUnits);
           fwrite(tmp,sizeof(char),strlen(tmp), profileFile);
           fwrite(DS_TAG_DEVICE_MAX_COMPUTE_UNITS_END, sizeof(char), strlen(DS_TAG_DEVICE_MAX_COMPUTE_UNITS_END), profileFile);
 
           fwrite(DS_TAG_DEVICE_MAX_CLOCK_FREQ, sizeof(char), strlen(DS_TAG_DEVICE_MAX_CLOCK_FREQ), profileFile);
-          sprintf(tmp,"%d",profile->devices[i].oclMaxClockFrequency);
+          (void) FormatLocaleString(tmp,sizeof(tmp),"%d",
+            profile->devices[i].oclMaxClockFrequency);
           fwrite(tmp,sizeof(char),strlen(tmp), profileFile);
           fwrite(DS_TAG_DEVICE_MAX_CLOCK_FREQ_END, sizeof(char), strlen(DS_TAG_DEVICE_MAX_CLOCK_FREQ_END), profileFile);
         }
@@ -2063,7 +2065,7 @@ static ds_status writeProfileToFile(ds_profile* profile, ds_score_serializer ser
       status = serializer(profile->devices+i, &serializedScore, &serializedScoreSize);
       if (status == DS_SUCCESS && serializedScore!=NULL && serializedScoreSize > 0) {
         fwrite(serializedScore, sizeof(char), serializedScoreSize, profileFile);
-        free(serializedScore);
+        RelinquishMagickMemory(serializedScore);
       }
       fwrite(DS_TAG_SCORE_END, sizeof(char), strlen(DS_TAG_SCORE_END), profileFile);
       fwrite(DS_TAG_DEVICE_END, sizeof(char), strlen(DS_TAG_DEVICE_END), profileFile);
@@ -2093,7 +2095,7 @@ static ds_status readProFile(const char* fileName, char** content, size_t* conte
   fseek(input, 0L, SEEK_END);
   size = ftell(input);
   rewind(input);
-  binary = (char*)malloc(size);
+  binary = (char*) AcquireQuantumMemory(1,size);
   if(binary == NULL) {
     status = DS_FILE_ERROR;
     goto cleanup;
@@ -2111,7 +2113,7 @@ cleanup:
   if (input != NULL) fclose(input);
   if (status != DS_SUCCESS
       && binary != NULL) {
-      free(binary);
+      RelinquishMagickMemory(binary);
       *content = NULL;
       *contentSize = 0;
   }
@@ -2353,7 +2355,7 @@ RestoreMSCWarning
     }
   }
 cleanup:
-  if (contentStart!=NULL) free(contentStart);
+  if (contentStart!=NULL) RelinquishMagickMemory(contentStart);
   return status;
 }
 
@@ -2484,7 +2486,7 @@ static ds_status AcceleratePerfEvaluator(ds_device *device,
       resizedImage=ResizeImage(unsharpedImage,640,480,LanczosFilter,1.0,
         exception);
 
-      /* 
+      /*
         We need this to get a proper performance benchmark, the operations
         are executed asynchronous.
       */
@@ -2515,12 +2517,12 @@ static ds_status AcceleratePerfEvaluator(ds_device *device,
   /* end of microbenchmark */
 
   if (device->score == NULL)
-    device->score=malloc(sizeof(AccelerateScoreType));
+    device->score= AcquireMagickMemory(sizeof(AccelerateScoreType));
 
   if (status != MagickFalse)
-    *(AccelerateScoreType*)device->score=readAccelerateTimer(&timer);
+    *(AccelerateScoreType*) device->score=readAccelerateTimer(&timer);
   else
-    *(AccelerateScoreType*)device->score=42;
+    *(AccelerateScoreType*) device->score=42;
 
   ReturnStatus(DS_SUCCESS);
 }
@@ -2529,8 +2531,9 @@ ds_status AccelerateScoreSerializer(ds_device* device, void** serializedScore, u
   if (device
      && device->score) {
     /* generate a string from the score */
-    char* s = (char*)malloc(sizeof(char)*256);
-    sprintf(s,"%.4f",*((AccelerateScoreType*)device->score));
+    char* s = (char*) AcquireQuantumMemory(256,sizeof(char));
+    (void) FormatLocaleString(s,256,"%.4f",*((AccelerateScoreType*)
+      device->score));
     *serializedScore = (void*)s;
     *serializedScoreSize = (unsigned int) strlen(s);
     return DS_SUCCESS;
@@ -2543,13 +2546,13 @@ ds_status AccelerateScoreSerializer(ds_device* device, void** serializedScore, u
 ds_status AccelerateScoreDeserializer(ds_device* device, const unsigned char* serializedScore, unsigned int serializedScoreSize) {
   if (device) {
     /* convert the string back to an int */
-    char* s = (char*)malloc(serializedScoreSize+1);
+    char* s = (char*) AcquireQuantumMemory(1,serializedScoreSize+1);
     memcpy(s, serializedScore, serializedScoreSize);
     s[serializedScoreSize] = (char)'\0';
-    device->score = malloc(sizeof(AccelerateScoreType));
+    device->score = AcquireMagickMemory(sizeof(AccelerateScoreType));
     *((AccelerateScoreType*)device->score) = (AccelerateScoreType)
       strtod(s, (char **) NULL);
-    free(s);
+    RelinquishMagickMemory(s);
     return DS_SUCCESS;
   }
   else {
@@ -2559,7 +2562,7 @@ ds_status AccelerateScoreDeserializer(ds_device* device, const unsigned char* se
 
 ds_status AccelerateScoreRelease(void* score) {
   if (score!=NULL) {
-    free(score);
+    RelinquishMagickMemory(score);
   }
   return DS_SUCCESS;
 }
@@ -2877,12 +2880,14 @@ const char* GetOpenCLCachedFilesDirectory() {
       if (home == (char *) NULL)
       {
         home=GetEnvironmentValue("XDG_CACHE_HOME");
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__MINGW32__)
         if (home == (char *) NULL)
           home=GetEnvironmentValue("LOCALAPPDATA");
         if (home == (char *) NULL)
           home=GetEnvironmentValue("APPDATA");
         if (home == (char *) NULL)
           home=GetEnvironmentValue("USERPROFILE");
+#endif
       }
 
       if (home != (char *) NULL)
@@ -3002,6 +3007,8 @@ void OpenCLLog(const char* message) {
 
 
       log = fopen(path, "ab");
+      if (log == (FILE *) NULL)
+        return;
       fwrite(message, sizeof(char), strlen(message), log);
       fwrite("\n", sizeof(char), 1, log);
 
