@@ -1,8 +1,8 @@
 /*
-  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
 
     https://imagemagick.org/script/license.php
@@ -18,6 +18,7 @@
 #ifndef MAGICKCORE_STRING_PRIVATE_H
 #define MAGICKCORE_STRING_PRIVATE_H
 
+#include <string.h>
 #include "magick/locale_.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -39,6 +40,45 @@ static inline double SiPrefixToDoubleInterval(const char *string,
   return(value);
 }
 
+static inline const char *StringLocateSubstring(const char *haystack,
+  const char *needle)
+{
+#if defined(MAGICKCORE_HAVE_STRCASESTR)
+  return(strcasestr(haystack,needle));
+#else
+  {
+    size_t
+      length_needle,
+      length_haystack;
+
+    size_t
+      i;
+
+    if (!haystack || !needle)
+      return(NULL);
+    length_needle=strlen(needle);
+    length_haystack=strlen(haystack)-length_needle+1;
+    for (i=0; i < length_haystack; i++)
+    {
+      size_t
+        j;
+
+      for (j=0; j < length_needle; j++)
+      {
+        unsigned char c1 = (unsigned char) haystack[i+j];
+        unsigned char c2 = (unsigned char) needle[j];
+        if (toupper((int) c1) != toupper((int) c2))
+          goto next;
+      }
+      return((char *) haystack+i);
+      next:
+       ;
+    }
+    return((char *) NULL);
+  }
+#endif
+}
+
 static inline double StringToDouble(const char *magick_restrict string,
   char **magick_restrict sentinal)
 {
@@ -62,17 +102,34 @@ static inline double StringToDoubleInterval(const char *string,
 
 static inline int StringToInteger(const char *magick_restrict value)
 {
+  if (value == (const char *) NULL)
+    return(0);
   return((int) strtol(value,(char **) NULL,10));
 }
 
 static inline long StringToLong(const char *magick_restrict value)
 {
+  if (value == (const char *) NULL)
+    return(0);
   return(strtol(value,(char **) NULL,10));
 }
 
+static inline size_t StringToSizeType(const char *string,const double interval)
+{
+  double
+    value;
+
+  value=SiPrefixToDoubleInterval(string,interval);
+  if (value >= (double) MagickULLConstant(~0))
+    return(~0UL);
+  return((size_t) value);
+}
+
 static inline unsigned long StringToUnsignedLong(
   const char *magick_restrict value)
 {
+  if (value == (const char *) NULL)
+    return(0);
   return(strtoul(value,(char **) NULL,10));
 }
 
