@@ -17,7 +17,7 @@
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the license.  You may  %
@@ -48,8 +48,10 @@
 #include "magick/locale_.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
+#include "magick/memory-private.h"
 #include "magick/nt-base.h"
 #include "magick/nt-base-private.h"
+#include "magick/policy.h"
 #include "magick/property.h"
 #include "magick/resource_.h"
 #include "magick/signature-private.h"
@@ -111,7 +113,7 @@ static const unsigned char
 %  An extended string is the string length, plus an extra MaxTextExtent space
 %  to allow for the string to be actively worked on.
 %
-%  The returned string shoud be freed using DestoryString().
+%  The returned string should be freed using DestroyString().
 %
 %  The format of the AcquireString method is:
 %
@@ -168,7 +170,7 @@ MagickExport char *AcquireString(const char *source)
 %
 */
 
-static StringInfo *AcquireStringInfoContainer()
+static StringInfo *AcquireStringInfoContainer(void)
 {
   StringInfo
     *string_info;
@@ -266,7 +268,7 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
 %  If source is a NULL pointer the destination string will be freed and set to
 %  a NULL pointer.  A pointer to the stored in the destination is also returned.
 %
-%  When finished the non-NULL string should be freed using DestoryString()
+%  When finished the non-NULL string should be freed using DestroyString()
 %  or using CloneString() with a NULL pointed for the source.
 %
 %  The format of the CloneString method is:
@@ -340,7 +342,7 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickCoreSignature);
   clone_info=AcquireStringInfo(string_info->length);
-  (void) strcpy(clone_info->path,string_info->path);
+  (void) CopyMagickString(clone_info->path,string_info->path,MagickPathExtent);
   (void) CloneString(&clone_info->name,string_info->name);
   if (string_info->length != 0)
     (void) memcpy(clone_info->datum,string_info->datum,string_info->length+1);
@@ -512,8 +514,8 @@ MagickExport MagickBooleanType ConcatenateString(
   length+=source_length;
   if (~length < MaxTextExtent)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
-  *destination=(char *) ResizeQuantumMemory(*destination,length+MaxTextExtent,
-    sizeof(**destination));
+  *destination=(char *) ResizeQuantumMemory(*destination,
+    OverAllocateMemory(length+MaxTextExtent),sizeof(**destination));
   if (*destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
   if (source_length != 0)
@@ -560,8 +562,20 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
   length=string_info->length;
   if (~length < source->length)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
-  SetStringInfoLength(string_info,length+source->length);
-  (void) memcpy(string_info->datum+length,source->datum,source->length);
+  length+=source->length;
+  if (~length < MagickPathExtent)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  if (string_info->datum == (unsigned char *) NULL)
+    string_info->datum=(unsigned char *) AcquireQuantumMemory(length+
+      MagickPathExtent,sizeof(*string_info->datum));
+  else
+    string_info->datum=(unsigned char *) ResizeQuantumMemory( 
+      string_info->datum,OverAllocateMemory(length+MagickPathExtent),
+      sizeof(*string_info->datum));
+  if (string_info->datum == (unsigned char *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  (void) memcpy(string_info->datum+string_info->length,source->datum,source->length);
+  string_info->length=length;
 }
 
 /*
@@ -684,7 +698,7 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
 %  copies the source string to that memory location.  A NULL string pointer
 %  will allocate an empty string containing just the NUL character.
 %
-%  When finished the string should be freed using DestoryString()
+%  When finished the string should be freed using DestroyString()
 %
 %  The format of the ConstantString method is:
 %
@@ -983,12 +997,33 @@ MagickExport char *EscapeString(const char *source,const char escape)
 MagickExport char *FileToString(const char *filename,const size_t extent,
   ExceptionInfo *exception)
 {
+  const char
+    *p;
+
   size_t
     length;
 
   assert(filename != (const char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
-  return((char *) FileToBlob(filename,extent,&length,exception));
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
+  p=filename;
+  if ((*filename == '@') && (strlen(filename) > 1))
+    {
+      MagickBooleanType
+        status;
+
+      status=IsRightsAuthorized(PathPolicyDomain,ReadPolicyRights,filename);
+      if (status == MagickFalse)
+        {
+          errno=EPERM;
+          (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+            "NotAuthorized","`%s'",filename);
+          return((char *) NULL);
+        }
+      p=filename+1;
+    }
+  return((char *) FileToBlob(p,extent,&length,exception));
 }
 
 /*
@@ -1025,8 +1060,9 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
     *string_info;
 
   assert(filename != (const char *) NULL);
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(exception != (ExceptionInfo *) NULL);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   string_info=AcquireStringInfoContainer();
   (void) CopyMagickString(string_info->path,filename,MaxTextExtent);
   string_info->datum=FileToBlob(filename,extent,&string_info->length,exception);
@@ -1070,10 +1106,6 @@ MagickExport StringInfo *FileToStringInfo(const char *filename,
 MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
   const MagickBooleanType bi,char *format)
 {
-  char
-    p[MaxTextExtent],
-    q[MaxTextExtent];
-
   const char
     **units;
 
@@ -1090,11 +1122,11 @@ MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
   static const char
     *bi_units[] =
     {
-      "", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi", "Yi", (char *) NULL
+      "", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi", "Yi", "Ri", "Qi", (char *) NULL
     },
     *traditional_units[] =
     {
-      "", "K", "M", "G", "T", "P", "E", "Z", "Y", (char *) NULL
+      "", "K", "M", "G", "T", "P", "E", "Z", "Y", "R", "Q", (char *) NULL
     };
 
   bytes=1000.0;
@@ -1109,10 +1141,9 @@ MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
 #else
   length=(double) size;
 #endif
-  (void) FormatLocaleString(p,MaxTextExtent,"%.*g",GetMagickPrecision(),
+  (void) FormatLocaleString(format,MaxTextExtent,"%.*g",GetMagickPrecision(),
     length);
-  (void) FormatLocaleString(q,MaxTextExtent,"%.20g",length);
-  if (strtod(p,(char **) NULL) == strtod(q,(char **) NULL))
+  if (strstr(format,"e+") == (char *) NULL)
     {
       count=FormatLocaleString(format,MaxTextExtent,"%.20g%sB",length,units[0]);
       return(count);
@@ -1291,18 +1322,18 @@ MagickExport const char *GetStringInfoPath(const StringInfo *string_info)
 %
 %  The format of the InterpretSiPrefixValue method is:
 %
-%      double InterpretSiPrefixValue(const char *value,char **sentinal)
+%      double InterpretSiPrefixValue(const char *value,char **sentinel)
 %
 %  A description of each parameter follows:
 %
 %    o value: the string value.
 %
-%    o sentinal:  if sentinal is not NULL, return a pointer to the character
+%    o sentinel:  if sentinel is not NULL, return a pointer to the character
 %      after the last character used in the conversion.
 %
 */
 MagickExport double InterpretSiPrefixValue(const char *magick_restrict string,
-  char **magick_restrict sentinal)
+  char **magick_restrict sentinel)
 {
   char
     *q;
@@ -1320,6 +1351,8 @@ MagickExport double InterpretSiPrefixValue(const char *magick_restrict string,
 
           switch ((int) ((unsigned char) *q))
           {
+            case 'q': e=(-30.0); break;
+            case 'r': e=(-27.0); break;
             case 'y': e=(-24.0); break;
             case 'z': e=(-21.0); break;
             case 'a': e=(-18.0); break;
@@ -1340,6 +1373,8 @@ MagickExport double InterpretSiPrefixValue(const char *magick_restrict string,
             case 'E': e=18.0; break;
             case 'Z': e=21.0; break;
             case 'Y': e=24.0; break;
+            case 'R': e=27.0; break;
+            case 'Q': e=30.0; break;
             default: e=0.0; break;
           }
           if (e >= MagickEpsilon)
@@ -1359,8 +1394,8 @@ MagickExport double InterpretSiPrefixValue(const char *magick_restrict string,
       if ((*q == 'B') || (*q == 'P'))
         q++;
     }
-  if (sentinal != (char **) NULL)
-    *sentinal=q;
+  if (sentinel != (char **) NULL)
+    *sentinel=q;
   return(value);
 }
 
@@ -2079,7 +2114,7 @@ MagickExport char **StringToArgv(const char *text,int *argc)
 %
 %  StringToArrayOfDoubles() converts a string of space or comma separated
 %  numbers into array of floating point numbers (doubles). Any number that
-%  failes to parse properly will produce a syntax error. As will two commas
+%  fails to parse properly will produce a syntax error. As will two commas
 %  without a  number between them.  However a final comma at the end will
 %  not be regarded as an error so as to simplify automatic list generation.
 %
@@ -2578,8 +2613,8 @@ MagickExport MagickBooleanType SubstituteString(char **string,
         */
         offset=(ssize_t) (p-(*string));
         extent=strlen(*string)+replace_extent-search_extent+1;
-        *string=(char *) ResizeQuantumMemory(*string,extent+MaxTextExtent,
-          sizeof(*p));
+        *string=(char *) ResizeQuantumMemory(*string,
+          OverAllocateMemory(extent+MaxTextExtent),sizeof(*p));
         if (*string == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
         p=(*string)+offset;
