@@ -17,7 +17,7 @@
 %                                 October 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -112,8 +112,11 @@ struct _ThresholdMap
 /*
   Static declarations.
 */
+#if MAGICKCORE_ZERO_CONFIGURATION_SUPPORT
+  #include "magick/threshold-map.h"
+#else
 static const char
-  *MinimalThresholdMap =
+  *BuiltinMap =
     "<?xml version=\"1.0\"?>"
     "<thresholds>"
     "  <threshold map=\"threshold\" alias=\"1x1\">"
@@ -130,6 +133,7 @@ static const char
     "    </levels>"
     "  </threshold>"
     "</thresholds>";
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -196,7 +200,7 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
 
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
@@ -275,18 +279,18 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       {
         if (u == (ssize_t) (width-1))
           {
-            channel_bias.red+=r[u].red;
-            channel_bias.green+=r[u].green;
-            channel_bias.blue+=r[u].blue;
-            channel_bias.opacity+=r[u].opacity;
+            channel_bias.red+=(MagickRealType) r[u].red;
+            channel_bias.green+=(MagickRealType) r[u].green;
+            channel_bias.blue+=(MagickRealType) r[u].blue;
+            channel_bias.opacity+=(MagickRealType) r[u].opacity;
             if (image->colorspace == CMYKColorspace)
               channel_bias.index=(MagickRealType)
                 GetPixelIndex(indexes+(r-p)+u);
           }
-        channel_sum.red+=r[u].red;
-        channel_sum.green+=r[u].green;
-        channel_sum.blue+=r[u].blue;
-        channel_sum.opacity+=r[u].opacity;
+        channel_sum.red+=(MagickRealType) r[u].red;
+        channel_sum.green+=(MagickRealType) r[u].green;
+        channel_sum.blue+=(MagickRealType) r[u].blue;
+        channel_sum.opacity+=(MagickRealType) r[u].opacity;
         if (image->colorspace == CMYKColorspace)
           channel_sum.index=(MagickRealType) GetPixelIndex(indexes+(r-p)+u);
       }
@@ -307,16 +311,16 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       channel_bias=zero;
       for (v=0; v < (ssize_t) height; v++)
       {
-        channel_bias.red+=r[0].red;
-        channel_bias.green+=r[0].green;
-        channel_bias.blue+=r[0].blue;
-        channel_bias.opacity+=r[0].opacity;
+        channel_bias.red+=(MagickRealType) r[0].red;
+        channel_bias.green+=(MagickRealType) r[0].green;
+        channel_bias.blue+=(MagickRealType) r[0].blue;
+        channel_bias.opacity+=(MagickRealType) r[0].opacity;
         if (image->colorspace == CMYKColorspace)
           channel_bias.index=(MagickRealType) GetPixelIndex(indexes+x+(r-p)+0);
-        channel_sum.red+=r[width-1].red;
-        channel_sum.green+=r[width-1].green;
-        channel_sum.blue+=r[width-1].blue;
-        channel_sum.opacity+=r[width-1].opacity;
+        channel_sum.red+=(MagickRealType) r[width-1].red;
+        channel_sum.green+=(MagickRealType) r[width-1].green;
+        channel_sum.blue+=(MagickRealType) r[width-1].blue;
+        channel_sum.opacity+=(MagickRealType) r[width-1].opacity;
         if (image->colorspace == CMYKColorspace)
           channel_sum.index=(MagickRealType) GetPixelIndex(indexes+x+(r-p)+
             width-1);
@@ -416,7 +420,7 @@ static double KapurThreshold(const Image *image,const double *histogram,
     threshold;
 
   /*
-    Compute optimal threshold from the entopy of the histogram.
+    Compute optimal threshold from the entropy of the histogram.
   */
   cumulative_histogram=(double *) AcquireQuantumMemory(MaxIntensity+1UL,
     sizeof(*cumulative_histogram));
@@ -573,7 +577,7 @@ static double OTSUThreshold(const Image *image,const double *histogram,
   return(100.0*threshold/MaxIntensity);
 }
 
-static double TriangleThreshold(const Image *image,const double *histogram)
+static double TriangleThreshold(const double *histogram)
 {
   double
     a,
@@ -601,7 +605,6 @@ static double TriangleThreshold(const Image *image,const double *histogram)
   /*
     Compute optimal threshold with triangle algorithm.
   */
-  magick_unreferenced(image);
   start=0;  /* find start bin, first bin not zero count */
   for (i=0; i <= (ssize_t) MaxIntensity; i++)
     if (histogram[i] > 0.0)
@@ -696,7 +699,7 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   */
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   histogram=(double *) AcquireQuantumMemory(MaxIntensity+1UL,
     sizeof(*histogram));
@@ -752,7 +755,7 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
     }
     case TriangleThresholdMethod:
     {
-      threshold=TriangleThreshold(image,histogram);
+      threshold=TriangleThreshold(histogram);
       break;
     }
   }
@@ -769,7 +772,7 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
   artifact=GetImageArtifact(image,"threshold:verbose");
   if (IsStringTrue(artifact) != MagickFalse)
     (void) FormatLocaleFile(stdout,"%.*g%%\n",GetMagickPrecision(),threshold);
-  return(BilevelImage(image,QuantumRange*threshold/100.0));
+  return(BilevelImage(image,(MagickRealType) QuantumRange*threshold/100.0));
 }
 
 /*
@@ -846,7 +849,7 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
@@ -1013,7 +1016,7 @@ MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (thresholds == (const char *) NULL)
     return(MagickTrue);
@@ -1036,11 +1039,11 @@ MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
     threshold.index=threshold.red;
   if ((flags & PercentValue) != 0)
     {
-      threshold.red*=(MagickRealType) (QuantumRange/100.0);
-      threshold.green*=(MagickRealType) (QuantumRange/100.0);
-      threshold.blue*=(MagickRealType) (QuantumRange/100.0);
-      threshold.opacity*=(MagickRealType) (QuantumRange/100.0);
-      threshold.index*=(MagickRealType) (QuantumRange/100.0);
+      threshold.red*=(MagickRealType) QuantumRange/100.0;
+      threshold.green*=(MagickRealType) QuantumRange/100.0;
+      threshold.blue*=(MagickRealType) QuantumRange/100.0;
+      threshold.opacity*=(MagickRealType) QuantumRange/100.0;
+      threshold.index*=(MagickRealType) QuantumRange/100.0;
     }
   if ((IsMagickGray(&threshold) == MagickFalse) &&
       (IsGrayColorspace(image->colorspace) != MagickFalse))
@@ -1175,7 +1178,7 @@ MagickExport MagickBooleanType ClampImageChannel(Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->storage_class == PseudoClass)
     {
@@ -1312,7 +1315,7 @@ MagickExport ThresholdMap *DestroyThresholdMap(ThresholdMap *map)
 %  GetThresholdMapFile() look for a given threshold map name or alias in the
 %  given XML file data, and return the allocated the map when found.
 %
-%  The format of the ListThresholdMaps method is:
+%  The format of the GetThresholdMapFile method is:
 %
 %      ThresholdMap *GetThresholdMap(const char *xml,const char *filename,
 %         const char *map_id,ExceptionInfo *exception)
@@ -1558,7 +1561,7 @@ MagickExport ThresholdMap *GetThresholdMap(const char *map_id,
   ThresholdMap
     *map;
 
-  map=GetThresholdMapFile(MinimalThresholdMap,"built-in",map_id,exception);
+  map=GetThresholdMapFile(BuiltinMap,"built-in",map_id,exception);
   if (map != (ThresholdMap *) NULL)
     return(map);
   options=GetConfigureOptions(ThresholdsFilename,exception);
@@ -1816,7 +1819,7 @@ MagickExport MagickBooleanType OrderedDitherImageChannel(Image *image,
 %      each level. While checker,8,8,4 will produce a 332 colormaped image
 %      with only a single checkerboard hash pattern (50% grey) between each
 %      color level, to basically double the number of color levels with
-%      a bare minimim of dithering.
+%      a bare minimum of dithering.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -1857,10 +1860,10 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (threshold_map == (const char *) NULL)
     return(MagickTrue);
   {
@@ -2015,27 +2018,28 @@ printf("DEBUG levels  r=%u g=%u b=%u a=%u i=%u\n",
               Opacity is inverted so 'off' represents transparent.
         */
         if (levels.red) {
-          t = (ssize_t) (QuantumScale*GetPixelRed(q)*(levels.red*d+1));
+          t = (ssize_t) (QuantumScale*(MagickRealType) GetPixelRed(q)*
+            (levels.red*d+1));
           l = t/d;  t = t-l*d;
           SetPixelRed(q,ClampToQuantum((MagickRealType)
             ((l+(t >= threshold))*(MagickRealType) QuantumRange/levels.red)));
         }
         if (levels.green) {
-          t = (ssize_t) (QuantumScale*GetPixelGreen(q)*
+          t = (ssize_t) (QuantumScale*(MagickRealType) GetPixelGreen(q)*
             (levels.green*d+1));
           l = t/d;  t = t-l*d;
           SetPixelGreen(q,ClampToQuantum((MagickRealType)
             ((l+(t >= threshold))*(MagickRealType) QuantumRange/levels.green)));
         }
         if (levels.blue) {
-          t = (ssize_t) (QuantumScale*GetPixelBlue(q)*
+          t = (ssize_t) (QuantumScale*(MagickRealType) GetPixelBlue(q)*
             (levels.blue*d+1));
           l = t/d;  t = t-l*d;
           SetPixelBlue(q,ClampToQuantum((MagickRealType)
             ((l+(t >= threshold))*(MagickRealType) QuantumRange/levels.blue)));
         }
         if (levels.opacity) {
-          t = (ssize_t) ((1.0-QuantumScale*GetPixelOpacity(q))*
+          t = (ssize_t) ((1.0-QuantumScale*(MagickRealType) GetPixelOpacity(q))*
             (levels.opacity*d+1));
           l = t/d;  t = t-l*d;
           SetPixelOpacity(q,ClampToQuantum((MagickRealType)
@@ -2043,7 +2047,7 @@ printf("DEBUG levels  r=%u g=%u b=%u a=%u i=%u\n",
             levels.opacity)));
         }
         if (levels.index) {
-          t = (ssize_t) (QuantumScale*GetPixelIndex(indexes+x)*
+          t = (ssize_t) (QuantumScale*(MagickRealType) GetPixelIndex(indexes+x)*
             (levels.index*d+1));
           l = t/d;  t = t-l*d;
           SetPixelIndex(indexes+x,ClampToQuantum((MagickRealType) ((l+
@@ -2111,7 +2115,7 @@ static inline Quantum PerceptibleThreshold(const Quantum quantum,
     sign;
 
   sign=(double) quantum < 0.0 ? -1.0 : 1.0;
-  if ((sign*quantum) >= epsilon)
+  if ((sign*(double) quantum) >= epsilon)
     return(quantum);
   return((Quantum) (sign*epsilon));
 }
@@ -2148,7 +2152,7 @@ MagickExport MagickBooleanType PerceptibleImageChannel(Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->storage_class == PseudoClass)
     {
@@ -2325,10 +2329,10 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (thresholds == (const char *) NULL)
     return(MagickTrue);
   GetMagickPixelPacket(image,&threshold);
@@ -2341,8 +2345,8 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
     max_threshold=min_threshold;
   if (strchr(thresholds,'%') != (char *) NULL)
     {
-      max_threshold*=(MagickRealType) (0.01*QuantumRange);
-      min_threshold*=(MagickRealType) (0.01*QuantumRange);
+      max_threshold*=0.01*(MagickRealType) QuantumRange;
+      min_threshold*=0.01*(MagickRealType) QuantumRange;
     }
   else
     if (((max_threshold == min_threshold) || (max_threshold == 1)) &&
@@ -2364,7 +2368,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
       if (AcquireImageColormap(image,2) == MagickFalse)
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
-      random_info=AcquireRandomInfoThreadSet();
+      random_info=AcquireRandomInfoTLS();
       image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
       key=GetRandomSecretKey(random_info[0]);
@@ -2412,8 +2416,8 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           else if (intensity > max_threshold)
             threshold.index=max_threshold;
           else
-            threshold.index=(MagickRealType)(QuantumRange*
-              GetPseudoRandomValue(random_info[id]));
+            threshold.index=(MagickRealType) QuantumRange*
+              (MagickRealType) GetPseudoRandomValue(random_info[id]);
           index=(IndexPacket) (intensity <= threshold.index ? 0 : 1);
           SetPixelIndex(indexes+x,index);
           SetPixelRGBO(q,image->colormap+(ssize_t) index);
@@ -2438,7 +2442,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           }
       }
       image_view=DestroyCacheView(image_view);
-      random_info=DestroyRandomInfoThreadSet(random_info);
+      random_info=DestroyRandomInfoTLS(random_info);
       return(status);
     }
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
@@ -2446,7 +2450,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
       InheritException(exception,&image->exception);
       return(MagickFalse);
     }
-  random_info=AcquireRandomInfoThreadSet();
+  random_info=AcquireRandomInfoTLS();
   image_view=AcquireAuthenticCacheView(image,exception);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   key=GetRandomSecretKey(random_info[0]);
@@ -2486,7 +2490,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
             if ((MagickRealType) GetPixelRed(q) > max_threshold)
               threshold.red=max_threshold;
             else
-              threshold.red=(MagickRealType) (QuantumRange*
+              threshold.red=(MagickRealType) ((MagickRealType) QuantumRange*
                 GetPseudoRandomValue(random_info[id]));
         }
       if ((channel & GreenChannel) != 0)
@@ -2497,7 +2501,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
             if ((MagickRealType) GetPixelGreen(q) > max_threshold)
               threshold.green=max_threshold;
             else
-              threshold.green=(MagickRealType) (QuantumRange*
+              threshold.green=(MagickRealType) ((MagickRealType) QuantumRange*
                 GetPseudoRandomValue(random_info[id]));
         }
       if ((channel & BlueChannel) != 0)
@@ -2508,7 +2512,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
             if ((MagickRealType) GetPixelBlue(q) > max_threshold)
               threshold.blue=max_threshold;
             else
-              threshold.blue=(MagickRealType) (QuantumRange*
+              threshold.blue=(MagickRealType) ((MagickRealType) QuantumRange*
                 GetPseudoRandomValue(random_info[id]));
         }
       if ((channel & OpacityChannel) != 0)
@@ -2519,7 +2523,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
             if ((MagickRealType) GetPixelOpacity(q) > max_threshold)
               threshold.opacity=max_threshold;
             else
-              threshold.opacity=(MagickRealType) (QuantumRange*
+              threshold.opacity=(MagickRealType) ((MagickRealType) QuantumRange*
                 GetPseudoRandomValue(random_info[id]));
         }
       if (((channel & IndexChannel) != 0) &&
@@ -2531,7 +2535,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
             if ((MagickRealType) GetPixelIndex(indexes+x) > max_threshold)
               threshold.index=max_threshold;
             else
-              threshold.index=(MagickRealType) (QuantumRange*
+              threshold.index=(MagickRealType) ((MagickRealType) QuantumRange*
                 GetPseudoRandomValue(random_info[id]));
         }
       if ((channel & RedChannel) != 0)
@@ -2569,7 +2573,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
       }
   }
   image_view=DestroyCacheView(image_view);
-  random_info=DestroyRandomInfoThreadSet(random_info);
+  random_info=DestroyRandomInfoTLS(random_info);
   return(status);
 }
 
@@ -2645,7 +2649,7 @@ MagickExport MagickBooleanType WhiteThresholdImageChannel(Image *image,
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (thresholds == (const char *) NULL)
     return(MagickTrue);
@@ -2668,11 +2672,11 @@ MagickExport MagickBooleanType WhiteThresholdImageChannel(Image *image,
     threshold.index=threshold.red;
   if ((flags & PercentValue) != 0)
     {
-      threshold.red*=(MagickRealType) (QuantumRange/100.0);
-      threshold.green*=(MagickRealType) (QuantumRange/100.0);
-      threshold.blue*=(MagickRealType) (QuantumRange/100.0);
-      threshold.opacity*=(MagickRealType) (QuantumRange/100.0);
-      threshold.index*=(MagickRealType) (QuantumRange/100.0);
+      threshold.red*=(MagickRealType) QuantumRange/100.0;
+      threshold.green*=(MagickRealType) QuantumRange/100.0;
+      threshold.blue*=(MagickRealType) QuantumRange/100.0;
+      threshold.opacity*=(MagickRealType) QuantumRange/100.0;
+      threshold.index*=(MagickRealType) QuantumRange/100.0;
     }
   if ((IsMagickGray(&threshold) == MagickFalse) &&
       (IsGrayColorspace(image->colorspace) != MagickFalse))
