@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.  You may
@@ -32,14 +32,10 @@ extern MagickPrivate MagickBooleanType
 static inline int MagickReadDirectory(DIR *directory,struct dirent *entry,
   struct dirent **result)
 {
-#if defined(MAGICKCORE_HAVE_READDIR_R)
-  return(readdir_r(directory,entry,result));
-#else
   (void) entry;
   errno=0;
   *result=readdir(directory);
   return(errno);
-#endif
 }
 
 /*
@@ -67,20 +63,22 @@ static inline wchar_t *create_wchar_path(const char *utf8)
 
       (void) FormatLocaleString(buffer,MaxTextExtent,"\\\\?\\%s",utf8);
       count+=4;
-      longPath=(wchar_t *) AcquireQuantumMemory(count,sizeof(*longPath));
+      longPath=(wchar_t *) NTAcquireQuantumMemory((size_t) count,
+        sizeof(*longPath));
       if (longPath == (wchar_t *) NULL)
         return((wchar_t *) NULL);
       count=MultiByteToWideChar(CP_UTF8,0,buffer,-1,longPath,count);
       if (count != 0)
-        count=GetShortPathNameW(longPath,shortPath,MAX_PATH);
+        count=(int) GetShortPathNameW(longPath,shortPath,MAX_PATH);
       longPath=(wchar_t *) RelinquishMagickMemory(longPath);
       if ((count < 5) || (count >= MAX_PATH))
         return((wchar_t *) NULL);
-      wideChar=(wchar_t *) AcquireQuantumMemory(count-3,sizeof(*wideChar));
+      wideChar=(wchar_t *) NTAcquireQuantumMemory((size_t) count-3,
+        sizeof(*wideChar));
       wcscpy(wideChar,shortPath+4);
       return(wideChar);
     }
-  wideChar=(wchar_t *) AcquireQuantumMemory(count,sizeof(*wideChar));
+  wideChar=(wchar_t *) NTAcquireQuantumMemory(count,sizeof(*wideChar));
   if (wideChar == (wchar_t *) NULL)
     return((wchar_t *) NULL);
   count=MultiByteToWideChar(CP_UTF8,0,utf8,-1,wideChar,count);
@@ -95,6 +93,8 @@ static inline wchar_t *create_wchar_path(const char *utf8)
 
 static inline int access_utf8(const char *path,int mode)
 {
+  if (path == (const char *) NULL)
+    return(-1);
 #if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__CYGWIN__)
   return(access(path,mode));
 #else
@@ -205,7 +205,8 @@ static inline FILE *popen_utf8(const char *command,const char *type)
   length=MultiByteToWideChar(CP_UTF8,0,command,-1,NULL,0);
   if (length == 0)
     return(file);
-  command_wide=(wchar_t *) AcquireQuantumMemory(length,sizeof(*command_wide));
+  command_wide=(wchar_t *) AcquireQuantumMemory((size_t) length,
+    sizeof(*command_wide));
   if (command_wide == (wchar_t *) NULL)
     return(file);
   length=MultiByteToWideChar(CP_UTF8,0,command,-1,command_wide,length);
