@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.  You may
@@ -33,7 +33,8 @@ typedef enum
   ExtensionPath,
   SubimagePath,
   CanonicalPath,
-  SubcanonicalPath
+  SubcanonicalPath,
+  BasePathSansCompressExtension
 } PathType;
 
 extern MagickExport char
