@@ -23,7 +23,7 @@
 %                                 August 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999 ImageMagick Studio LLC, a non-profit organization           %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -261,6 +261,9 @@ WandExport MagickBooleanType MagickClearException(MagickWand *wand)
 %  MagickGetException() returns the severity, reason, and description of any
 %  error that occurs when using other methods in this API.
 %
+%  Use RelinquishMagickMemory() to free the description when its no longer in
+%  use.
+%
 %  The format of the MagickGetException method is:
 %
 %      char *MagickGetException(const MagickWand *wand,ExceptionType *severity)
@@ -789,7 +792,8 @@ WandExport char **MagickQueryFormats(const char *pattern,
 */
 WandExport void *MagickRelinquishMemory(void *memory)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   return(RelinquishMagickMemory(memory));
 }
 
@@ -1021,7 +1025,29 @@ WandExport void MagickWandGenesis(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  MagickWandTerminus() terminates the MagickWand environment.
+%  MagickWandTerminus() is a function in the ImageMagick library that is
+%  used to clean up and release resources when shutting down an application
+%  that uses ImageMagick. This function should be called in the primary thread
+%  of the application's process during the shutdown process. It's crucial that
+%  this function is invoked only after any threads that are using ImageMagick
+%  functions have terminated.
+%
+%  ImageMagick might internally use threads via OpenMP (a method for parallel
+%  programming). As a result, it's important to ensure that any function calls
+%  into ImageMagick have completed before calling MagickWandTerminus(). This
+%  prevents issues with OpenMP worker threads accessing resources that are
+%  destroyed by this termination function.
+%
+%  If OpenMP is being used (starting from version 5.0), the OpenMP
+%  implementation itself handles starting and stopping worker threads and
+%  allocating and freeing resources using its own methods. This means that
+%  after calling MagickWandTerminus(), some OpenMP resources and worker
+%  threads might still remain allocated. To address this, the function
+%  omp_pause_resource_all(omp_pause_hard) can be invoked. This function,
+%  introduced in OpenMP version 5.0, ensures that any resources allocated by
+%  OpenMP (such as threads and thread-specific memory) are freed. It's
+%  recommended to call this function after MagickWandTerminus() has completed
+%  its execution.
 %
 %  The format of the MagickWandTerminus method is:
 %
